import typing, clr, abc
from System import IDisposable, Attribute, EventArgs, Array_1, Exception, Guid, DateTime, TimeSpan, Func_1
from System.Collections.Generic import IDictionary_2, IEnumerable_1
from System.Collections import IDictionary
from System.Reflection import MethodBase
from System.Collections.ObjectModel import ReadOnlyCollection_1

class DiagnosticCounter(IDisposable, abc.ABC):
    @property
    def DisplayName(self) -> str: ...
    @DisplayName.setter
    def DisplayName(self, value: str) -> str: ...
    @property
    def DisplayUnits(self) -> str: ...
    @DisplayUnits.setter
    def DisplayUnits(self, value: str) -> str: ...
    @property
    def EventSource(self) -> EventSource: ...
    @property
    def Name(self) -> str: ...
    def AddMetadata(self, key: str, value: str) -> None: ...
    def Dispose(self) -> None: ...


class EventActivityOptions(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    None_ : EventActivityOptions # 0
    Disable : EventActivityOptions # 2
    Recursive : EventActivityOptions # 4
    Detachable : EventActivityOptions # 8


class EventAttribute(Attribute):
    def __init__(self, eventId: int) -> None: ...
    @property
    def ActivityOptions(self) -> EventActivityOptions: ...
    @ActivityOptions.setter
    def ActivityOptions(self, value: EventActivityOptions) -> EventActivityOptions: ...
    @property
    def Channel(self) -> EventChannel: ...
    @Channel.setter
    def Channel(self, value: EventChannel) -> EventChannel: ...
    @property
    def EventId(self) -> int: ...
    @EventId.setter
    def EventId(self, value: int) -> int: ...
    @property
    def Keywords(self) -> EventKeywords: ...
    @Keywords.setter
    def Keywords(self, value: EventKeywords) -> EventKeywords: ...
    @property
    def Level(self) -> EventLevel: ...
    @Level.setter
    def Level(self, value: EventLevel) -> EventLevel: ...
    @property
    def Message(self) -> str: ...
    @Message.setter
    def Message(self, value: str) -> str: ...
    @property
    def Opcode(self) -> EventOpcode: ...
    @Opcode.setter
    def Opcode(self, value: EventOpcode) -> EventOpcode: ...
    @property
    def Tags(self) -> EventTags: ...
    @Tags.setter
    def Tags(self, value: EventTags) -> EventTags: ...
    @property
    def Task(self) -> EventTask: ...
    @Task.setter
    def Task(self, value: EventTask) -> EventTask: ...
    @property
    def TypeId(self) -> typing.Any: ...
    @property
    def Version(self) -> int: ...
    @Version.setter
    def Version(self, value: int) -> int: ...


class EventChannel(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    None_ : EventChannel # 0
    Admin : EventChannel # 16
    Operational : EventChannel # 17
    Analytic : EventChannel # 18
    Debug : EventChannel # 19


class EventCommand(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Update : EventCommand # 0
    Disable : EventCommand # -3
    Enable : EventCommand # -2
    SendManifest : EventCommand # -1


class EventCommandEventArgs(EventArgs):
    @property
    def Arguments(self) -> IDictionary_2[str, str]: ...
    @Arguments.setter
    def Arguments(self, value: IDictionary_2[str, str]) -> IDictionary_2[str, str]: ...
    @property
    def Command(self) -> EventCommand: ...
    @Command.setter
    def Command(self, value: EventCommand) -> EventCommand: ...
    def DisableEvent(self, eventId: int) -> bool: ...
    def EnableEvent(self, eventId: int) -> bool: ...


class EventCounter(DiagnosticCounter):
    def __init__(self, name: str, eventSource: EventSource) -> None: ...
    @property
    def DisplayName(self) -> str: ...
    @DisplayName.setter
    def DisplayName(self, value: str) -> str: ...
    @property
    def DisplayUnits(self) -> str: ...
    @DisplayUnits.setter
    def DisplayUnits(self, value: str) -> str: ...
    @property
    def EventSource(self) -> EventSource: ...
    @property
    def Name(self) -> str: ...
    def ToString(self) -> str: ...
    # Skipped WriteMetric due to it being static, abstract and generic.

    WriteMetric : WriteMetric_MethodGroup
    class WriteMetric_MethodGroup:
        def __call__(self, value: float) -> None:...
        # Method WriteMetric(value : Double) was skipped since it collides with above method



class EventDataAttribute(Attribute):
    def __init__(self) -> None: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def TypeId(self) -> typing.Any: ...


class EventFieldAttribute(Attribute):
    def __init__(self) -> None: ...
    @property
    def Format(self) -> EventFieldFormat: ...
    @Format.setter
    def Format(self, value: EventFieldFormat) -> EventFieldFormat: ...
    @property
    def Tags(self) -> EventFieldTags: ...
    @Tags.setter
    def Tags(self, value: EventFieldTags) -> EventFieldTags: ...
    @property
    def TypeId(self) -> typing.Any: ...


class EventFieldFormat(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Default : EventFieldFormat # 0
    String : EventFieldFormat # 2
    Boolean : EventFieldFormat # 3
    Hexadecimal : EventFieldFormat # 4
    Xml : EventFieldFormat # 11
    Json : EventFieldFormat # 12
    HResult : EventFieldFormat # 15


class EventFieldTags(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    None_ : EventFieldTags # 0


class EventIgnoreAttribute(Attribute):
    def __init__(self) -> None: ...
    @property
    def TypeId(self) -> typing.Any: ...


class EventKeywords(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    None_ : EventKeywords # 0
    MicrosoftTelemetry : EventKeywords # 562949953421312
    WdiContext : EventKeywords # 562949953421312
    WdiDiagnostic : EventKeywords # 1125899906842624
    Sqm : EventKeywords # 2251799813685248
    AuditFailure : EventKeywords # 4503599627370496
    CorrelationHint : EventKeywords # 4503599627370496
    AuditSuccess : EventKeywords # 9007199254740992
    EventLogClassic : EventKeywords # 36028797018963968
    All : EventKeywords # -1


class EventLevel(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    LogAlways : EventLevel # 0
    Critical : EventLevel # 1
    Error : EventLevel # 2
    Warning : EventLevel # 3
    Informational : EventLevel # 4
    Verbose : EventLevel # 5


class EventListener(IDisposable, abc.ABC):
    def DisableEvents(self, eventSource: EventSource) -> None: ...
    def Dispose(self) -> None: ...
    # Skipped EnableEvents due to it being static, abstract and generic.

    EnableEvents : EnableEvents_MethodGroup
    class EnableEvents_MethodGroup:
        @typing.overload
        def __call__(self, eventSource: EventSource, level: EventLevel) -> None:...
        @typing.overload
        def __call__(self, eventSource: EventSource, level: EventLevel, matchAnyKeyword: EventKeywords) -> None:...
        @typing.overload
        def __call__(self, eventSource: EventSource, level: EventLevel, matchAnyKeyword: EventKeywords, arguments: IDictionary_2[str, str]) -> None:...



class EventManifestOptions(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    None_ : EventManifestOptions # 0
    Strict : EventManifestOptions # 1
    AllCultures : EventManifestOptions # 2
    OnlyIfNeededForRegistration : EventManifestOptions # 4
    AllowEventSourceOverride : EventManifestOptions # 8


class EventOpcode(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Info : EventOpcode # 0
    Start : EventOpcode # 1
    Stop : EventOpcode # 2
    DataCollectionStart : EventOpcode # 3
    DataCollectionStop : EventOpcode # 4
    Extension : EventOpcode # 5
    Reply : EventOpcode # 6
    Resume : EventOpcode # 7
    Suspend : EventOpcode # 8
    Send : EventOpcode # 9
    Receive : EventOpcode # 240


class EventSource(IDisposable):
    @typing.overload
    def __init__(self, eventSourceName: str) -> None: ...
    @typing.overload
    def __init__(self, eventSourceName: str, config: EventSourceSettings) -> None: ...
    @typing.overload
    def __init__(self, eventSourceName: str, config: EventSourceSettings, traits: Array_1[str]) -> None: ...
    @property
    def ConstructionException(self) -> Exception: ...
    @classmethod
    @property
    def CurrentThreadActivityId(cls) -> Guid: ...
    @property
    def Guid(self) -> Guid: ...
    @property
    def Name(self) -> str: ...
    @property
    def Settings(self) -> EventSourceSettings: ...
    def Dispose(self) -> None: ...
    @staticmethod
    def GetGuid(eventSourceType: typing.Type[typing.Any]) -> Guid: ...
    @staticmethod
    def GetName(eventSourceType: typing.Type[typing.Any]) -> str: ...
    @staticmethod
    def GetSources() -> IEnumerable_1[EventSource]: ...
    def GetTrait(self, key: str) -> str: ...
    @staticmethod
    def SendCommand(eventSource: EventSource, command: EventCommand, commandArguments: IDictionary_2[str, str]) -> None: ...
    def ToString(self) -> str: ...
    # Skipped GenerateManifest due to it being static, abstract and generic.

    GenerateManifest : GenerateManifest_MethodGroup
    class GenerateManifest_MethodGroup:
        @typing.overload
        def __call__(self, eventSourceType: typing.Type[typing.Any], assemblyPathToIncludeInManifest: str) -> str:...
        @typing.overload
        def __call__(self, eventSourceType: typing.Type[typing.Any], assemblyPathToIncludeInManifest: str, flags: EventManifestOptions) -> str:...

    # Skipped IsEnabled due to it being static, abstract and generic.

    IsEnabled : IsEnabled_MethodGroup
    class IsEnabled_MethodGroup:
        @typing.overload
        def __call__(self) -> bool:...
        @typing.overload
        def __call__(self, level: EventLevel, keywords: EventKeywords) -> bool:...
        @typing.overload
        def __call__(self, level: EventLevel, keywords: EventKeywords, channel: EventChannel) -> bool:...

    # Skipped SetCurrentThreadActivityId due to it being static, abstract and generic.

    SetCurrentThreadActivityId : SetCurrentThreadActivityId_MethodGroup
    class SetCurrentThreadActivityId_MethodGroup:
        @typing.overload
        def __call__(self, activityId: Guid) -> None:...
        @typing.overload
        def __call__(self, activityId: Guid, oldActivityThatWillContinue: clr.Reference[Guid]) -> None:...

    # Skipped Write due to it being static, abstract and generic.

    Write : Write_MethodGroup
    class Write_MethodGroup:
        def __getitem__(self, t:typing.Type[Write_1_T1]) -> Write_1[Write_1_T1]: ...

        Write_1_T1 = typing.TypeVar('Write_1_T1')
        class Write_1(typing.Generic[Write_1_T1]):
            Write_1_T = EventSource.Write_MethodGroup.Write_1_T1
            @typing.overload
            def __call__(self, eventName: str, data: Write_1_T) -> None:...
            @typing.overload
            def __call__(self, eventName: str, options: EventSourceOptions, data: Write_1_T) -> None:...
            @typing.overload
            def __call__(self, eventName: str, options: clr.Reference[EventSourceOptions], data: clr.Reference[Write_1_T]) -> None:...
            @typing.overload
            def __call__(self, eventName: str, options: clr.Reference[EventSourceOptions], activityId: clr.Reference[Guid], relatedActivityId: clr.Reference[Guid], data: clr.Reference[Write_1_T]) -> None:...

        @typing.overload
        def __call__(self, eventName: str) -> None:...
        @typing.overload
        def __call__(self, eventName: str, options: EventSourceOptions) -> None:...


    class EventSourcePrimitive:
        # Operator not supported op_Implicit(value: Single)
        # Operator not supported op_Implicit(value: Double)
        # Operator not supported op_Implicit(value: Byte)
        # Operator not supported op_Implicit(value: Int16)
        # Operator not supported op_Implicit(value: Int32)
        # Operator not supported op_Implicit(value: Int64)
        # Operator not supported op_Implicit(value: SByte)
        # Operator not supported op_Implicit(value: UInt16)
        # Operator not supported op_Implicit(value: UInt32)
        # Operator not supported op_Implicit(value: UInt64)
        # Operator not supported op_Implicit(value: Char)
        # Operator not supported op_Implicit(value: UIntPtr)
        # Operator not supported op_Implicit(value: IntPtr)
        # Operator not supported op_Implicit(value: Decimal)
        # Operator not supported op_Implicit(value: Byte[])
        # Operator not supported op_Implicit(value: Boolean)
        # Operator not supported op_Implicit(value: String)
        # Operator not supported op_Implicit(value: Guid)
        # Operator not supported op_Implicit(value: DateTime)
        # Operator not supported op_Implicit(value: Enum)
        pass



class EventSourceAttribute(Attribute):
    def __init__(self) -> None: ...
    @property
    def Guid(self) -> str: ...
    @Guid.setter
    def Guid(self, value: str) -> str: ...
    @property
    def LocalizationResources(self) -> str: ...
    @LocalizationResources.setter
    def LocalizationResources(self, value: str) -> str: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def TypeId(self) -> typing.Any: ...


class EventSourceCreatedEventArgs(EventArgs):
    def __init__(self) -> None: ...
    @property
    def EventSource(self) -> EventSource: ...
    @EventSource.setter
    def EventSource(self, value: EventSource) -> EventSource: ...


class EventSourceException(Exception):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, message: str) -> None: ...
    @typing.overload
    def __init__(self, message: str, innerException: Exception) -> None: ...
    @property
    def Data(self) -> IDictionary: ...
    @property
    def HelpLink(self) -> str: ...
    @HelpLink.setter
    def HelpLink(self, value: str) -> str: ...
    @property
    def HResult(self) -> int: ...
    @HResult.setter
    def HResult(self, value: int) -> int: ...
    @property
    def InnerException(self) -> Exception: ...
    @property
    def Message(self) -> str: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> str: ...
    @property
    def StackTrace(self) -> str: ...
    @property
    def TargetSite(self) -> MethodBase: ...


class EventSourceOptions:
    @property
    def ActivityOptions(self) -> EventActivityOptions: ...
    @ActivityOptions.setter
    def ActivityOptions(self, value: EventActivityOptions) -> EventActivityOptions: ...
    @property
    def Keywords(self) -> EventKeywords: ...
    @Keywords.setter
    def Keywords(self, value: EventKeywords) -> EventKeywords: ...
    @property
    def Level(self) -> EventLevel: ...
    @Level.setter
    def Level(self, value: EventLevel) -> EventLevel: ...
    @property
    def Opcode(self) -> EventOpcode: ...
    @Opcode.setter
    def Opcode(self, value: EventOpcode) -> EventOpcode: ...
    @property
    def Tags(self) -> EventTags: ...
    @Tags.setter
    def Tags(self, value: EventTags) -> EventTags: ...


class EventSourceSettings(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Default : EventSourceSettings # 0
    ThrowOnEventWriteErrors : EventSourceSettings # 1
    EtwManifestEventFormat : EventSourceSettings # 4
    EtwSelfDescribingEventFormat : EventSourceSettings # 8


class EventTags(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    None_ : EventTags # 0


class EventTask(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    None_ : EventTask # 0


class EventWrittenEventArgs(EventArgs):
    @property
    def ActivityId(self) -> Guid: ...
    @property
    def Channel(self) -> EventChannel: ...
    @property
    def EventId(self) -> int: ...
    @property
    def EventName(self) -> str: ...
    @EventName.setter
    def EventName(self, value: str) -> str: ...
    @property
    def EventSource(self) -> EventSource: ...
    @property
    def Keywords(self) -> EventKeywords: ...
    @Keywords.setter
    def Keywords(self, value: EventKeywords) -> EventKeywords: ...
    @property
    def Level(self) -> EventLevel: ...
    @Level.setter
    def Level(self, value: EventLevel) -> EventLevel: ...
    @property
    def Message(self) -> str: ...
    @Message.setter
    def Message(self, value: str) -> str: ...
    @property
    def Opcode(self) -> EventOpcode: ...
    @Opcode.setter
    def Opcode(self, value: EventOpcode) -> EventOpcode: ...
    @property
    def OSThreadId(self) -> int: ...
    @OSThreadId.setter
    def OSThreadId(self, value: int) -> int: ...
    @property
    def Payload(self) -> ReadOnlyCollection_1[typing.Any]: ...
    @Payload.setter
    def Payload(self, value: ReadOnlyCollection_1[typing.Any]) -> ReadOnlyCollection_1[typing.Any]: ...
    @property
    def PayloadNames(self) -> ReadOnlyCollection_1[str]: ...
    @PayloadNames.setter
    def PayloadNames(self, value: ReadOnlyCollection_1[str]) -> ReadOnlyCollection_1[str]: ...
    @property
    def RelatedActivityId(self) -> Guid: ...
    @property
    def Tags(self) -> EventTags: ...
    @Tags.setter
    def Tags(self, value: EventTags) -> EventTags: ...
    @property
    def Task(self) -> EventTask: ...
    @property
    def TimeStamp(self) -> DateTime: ...
    @TimeStamp.setter
    def TimeStamp(self, value: DateTime) -> DateTime: ...
    @property
    def Version(self) -> int: ...


class IncrementingEventCounter(DiagnosticCounter):
    def __init__(self, name: str, eventSource: EventSource) -> None: ...
    @property
    def DisplayName(self) -> str: ...
    @DisplayName.setter
    def DisplayName(self, value: str) -> str: ...
    @property
    def DisplayRateTimeScale(self) -> TimeSpan: ...
    @DisplayRateTimeScale.setter
    def DisplayRateTimeScale(self, value: TimeSpan) -> TimeSpan: ...
    @property
    def DisplayUnits(self) -> str: ...
    @DisplayUnits.setter
    def DisplayUnits(self, value: str) -> str: ...
    @property
    def EventSource(self) -> EventSource: ...
    @property
    def Name(self) -> str: ...
    def Increment(self, increment: float = ...) -> None: ...
    def ToString(self) -> str: ...


class IncrementingPollingCounter(DiagnosticCounter):
    def __init__(self, name: str, eventSource: EventSource, totalValueProvider: Func_1[float]) -> None: ...
    @property
    def DisplayName(self) -> str: ...
    @DisplayName.setter
    def DisplayName(self, value: str) -> str: ...
    @property
    def DisplayRateTimeScale(self) -> TimeSpan: ...
    @DisplayRateTimeScale.setter
    def DisplayRateTimeScale(self, value: TimeSpan) -> TimeSpan: ...
    @property
    def DisplayUnits(self) -> str: ...
    @DisplayUnits.setter
    def DisplayUnits(self, value: str) -> str: ...
    @property
    def EventSource(self) -> EventSource: ...
    @property
    def Name(self) -> str: ...
    def ToString(self) -> str: ...


class NonEventAttribute(Attribute):
    def __init__(self) -> None: ...
    @property
    def TypeId(self) -> typing.Any: ...


class PollingCounter(DiagnosticCounter):
    def __init__(self, name: str, eventSource: EventSource, metricProvider: Func_1[float]) -> None: ...
    @property
    def DisplayName(self) -> str: ...
    @DisplayName.setter
    def DisplayName(self, value: str) -> str: ...
    @property
    def DisplayUnits(self) -> str: ...
    @DisplayUnits.setter
    def DisplayUnits(self, value: str) -> str: ...
    @property
    def EventSource(self) -> EventSource: ...
    @property
    def Name(self) -> str: ...
    def ToString(self) -> str: ...




from PySimultan2.src.PySimultan2.data_model import DataModel

from PySimultan2.tests import resources
from PySimultan2.src.PySimultan2.geometry.geometry_base import (GeometryModel, SimultanLayer, SimultanVertex,
                                                                SimultanEdge, SimultanEdgeLoop, SimultanFace,
                                                                SimultanVolume)

try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 importlib_resources.
    import importlib_resources as pkg_resources


with pkg_resources.path(resources, 'new_geometry_test.simultan') as r_path:
    project_path = str(r_path)


data_model = DataModel.create_new_project(project_path=project_path,
                                          user_name='admin',
                                          password='admin')


def create_geometry_model(name='new_geometry_test'):
    return GeometryModel(name=name,
                         data_model=data_model)


def test_create_layer(geo_model):
    new_layer = SimultanLayer(geometry_model=geo_model,
                              data_model=data_model,
                              name='new_layer')

    assert isinstance(new_layer, SimultanLayer)

    return new_layer


def create_vertex(layer, x, y, z, name, geo_model):
    new_vertex = SimultanVertex(geometry_model=geo_model,
                                x=x,
                                y=y,
                                z=z,
                                data_model=data_model,
                                name=name,
                                layer=layer)
    assert isinstance(new_vertex, SimultanVertex)
    return new_vertex


def create_edge(layer, vertices, name, geo_model):
    new_edge = SimultanEdge(geometry_model=geo_model,
                            data_model=data_model,
                            name=name,
                            layer=layer,
                            vertices=vertices)
    assert isinstance(new_edge, SimultanEdge)
    return new_edge


def create_edge_loop(layer, edges, name, geo_model):
    new_edge_loop = SimultanEdgeLoop(geometry_model=geo_model,
                                     data_model=data_model,
                                     name=name,
                                     layer=layer,
                                     edges=edges)
    assert isinstance(new_edge_loop, SimultanEdgeLoop)
    return new_edge_loop


def create_face(layer, edge_loop, name, geo_model):
    new_face = SimultanFace(geometry_model=geo_model,
                            data_model=data_model,
                            name=name,
                            layer=layer,
                            edge_loop=edge_loop)
    assert isinstance(new_face, SimultanFace)
    return new_face


def test_create_cube(layer, geo_model):
    # create a cube
    vertex_pos = [(0, 0, 0), (1, 0, 0), (1, 1, 0), (0, 1, 0),
                  (0, 0, 1), (1, 0, 1), (1, 1, 1), (0, 1, 1)]

    vertices = []
    for i, pos in enumerate(vertex_pos):
        vertices.append(create_vertex(layer,
                                      *pos,
                                      'new_vertex_{}'.format(i),
                                      geo_model
                                      )
                        )

    edges_vertices = [(vertices[0], vertices[1]),
                      (vertices[1], vertices[2]),
                      (vertices[2], vertices[3]),
                      (vertices[3], vertices[0]),
                      (vertices[4], vertices[5]),
                      (vertices[5], vertices[6]),
                      (vertices[6], vertices[7]),
                      (vertices[7], vertices[4]),
                      (vertices[0], vertices[4]),
                      (vertices[1], vertices[5]),
                      (vertices[2], vertices[6]),
                      (vertices[3], vertices[7])
                      ]
    edges = []

    for i, edge_vertices in enumerate(edges_vertices):
        edges.append(create_edge(layer,
                                 edge_vertices,
                                 'new_edge_{}'.format(i),
                                 geo_model
                                 ))

    # create 6 edge loops as boundary for the cube
    edge_loop_edges = [
        # Front face loop
        [edges[0], edges[1], edges[2], edges[3]],
        # Back face loop
        [edges[4], edges[5], edges[6], edges[7]],
        # Top face loop
        [edges[1], edges[10], edges[5], edges[9]],
        # Bottom face loop
        [edges[3], edges[11], edges[7], edges[8]],
        # Left face loop
        [edges[0], edges[9], edges[4], edges[8]],
        # Right face loop
        [edges[2], edges[11], edges[6], edges[10]]
    ]

    edge_loops = []
    for i, edge_loop_edge in enumerate(edge_loop_edges):
        edge_loops.append(create_edge_loop(layer,
                                           edge_loop_edge,
                                           'new_edge_loop_{}'.format(i),
                                           geo_model),
                          )

    # create faces
    faces = []
    for i, edge_loop in enumerate(edge_loops):
        faces.append(create_face(layer,
                                 edge_loop,
                                 'new_face_{}'.format(i),
                                 geo_model
                                 )
                     )

    # create a volume
    volume = SimultanVolume(geometry_model=geo_model,
                            data_model=data_model,
                            name='new_volume',
                            layer=layer,
                            faces=faces)

    assert isinstance(volume, SimultanVolume)


if __name__ == '__main__':
    geo_model = create_geometry_model(name='new_geometry_test')
    layer = test_create_layer(geo_model)
    test_create_cube(layer, geo_model)
    data_model.save()
    data_model.cleanup()
    print('All tests passed!')



import typing, abc
from System import Attribute

class Cer(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    None_ : Cer # 0
    MayFail : Cer # 1
    Success : Cer # 2


class Consistency(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    MayCorruptProcess : Consistency # 0
    MayCorruptAppDomain : Consistency # 1
    MayCorruptInstance : Consistency # 2
    WillNotCorruptState : Consistency # 3


class CriticalFinalizerObject(abc.ABC):
    pass


class PrePrepareMethodAttribute(Attribute):
    def __init__(self) -> None: ...
    @property
    def TypeId(self) -> typing.Any: ...


class ReliabilityContractAttribute(Attribute):
    def __init__(self, consistencyGuarantee: Consistency, cer: Cer) -> None: ...
    @property
    def Cer(self) -> Cer: ...
    @property
    def ConsistencyGuarantee(self) -> Consistency: ...
    @property
    def TypeId(self) -> typing.Any: ...




import typing
from System import Attribute, IFormatProvider, Array_1, ReadOnlyMemory_1
from System.IO import TextWriter
from System.Text import Encoding, StringBuilder
from System.Threading.Tasks import ValueTask, Task
from System.Threading import CancellationToken

class GeneratedCodeAttribute(Attribute):
    def __init__(self, tool: str, version: str) -> None: ...
    @property
    def Tool(self) -> str: ...
    @property
    def TypeId(self) -> typing.Any: ...
    @property
    def Version(self) -> str: ...


class IndentedTextWriter(TextWriter):
    @typing.overload
    def __init__(self, writer: TextWriter) -> None: ...
    @typing.overload
    def __init__(self, writer: TextWriter, tabString: str) -> None: ...
    DefaultTabString : str
    @property
    def Encoding(self) -> Encoding: ...
    @property
    def FormatProvider(self) -> IFormatProvider: ...
    @property
    def Indent(self) -> int: ...
    @Indent.setter
    def Indent(self, value: int) -> int: ...
    @property
    def InnerWriter(self) -> TextWriter: ...
    @property
    def NewLine(self) -> str: ...
    @NewLine.setter
    def NewLine(self, value: str) -> str: ...
    def Close(self) -> None: ...
    def DisposeAsync(self) -> ValueTask: ...
    def Flush(self) -> None: ...
    def WriteLineNoTabs(self, s: str) -> None: ...
    def WriteLineNoTabsAsync(self, s: str) -> Task: ...
    # Skipped FlushAsync due to it being static, abstract and generic.

    FlushAsync : FlushAsync_MethodGroup
    class FlushAsync_MethodGroup:
        @typing.overload
        def __call__(self) -> Task:...
        @typing.overload
        def __call__(self, cancellationToken: CancellationToken) -> Task:...

    # Skipped Write due to it being static, abstract and generic.

    Write : Write_MethodGroup
    class Write_MethodGroup:
        @typing.overload
        def __call__(self, value: float) -> None:...
        # Method Write(value : Single) was skipped since it collides with above method
        @typing.overload
        def __call__(self, s: str) -> None:...
        # Method Write(value : Char) was skipped since it collides with above method
        # Method Write(value : Int32) was skipped since it collides with above method
        # Method Write(value : Int64) was skipped since it collides with above method
        @typing.overload
        def __call__(self, buffer: Array_1[str]) -> None:...
        # Method Write(value : Boolean) was skipped since it collides with above method
        @typing.overload
        def __call__(self, value: typing.Any) -> None:...
        @typing.overload
        def __call__(self, format: str, arg: Array_1[typing.Any]) -> None:...
        @typing.overload
        def __call__(self, format: str, arg0: typing.Any) -> None:...
        @typing.overload
        def __call__(self, buffer: Array_1[str], index: int, count: int) -> None:...
        @typing.overload
        def __call__(self, format: str, arg0: typing.Any, arg1: typing.Any) -> None:...

    # Skipped WriteAsync due to it being static, abstract and generic.

    WriteAsync : WriteAsync_MethodGroup
    class WriteAsync_MethodGroup:
        @typing.overload
        def __call__(self, value: str) -> Task:...
        # Method WriteAsync(value : String) was skipped since it collides with above method
        @typing.overload
        def __call__(self, buffer: ReadOnlyMemory_1[str], cancellationToken: CancellationToken = ...) -> Task:...
        @typing.overload
        def __call__(self, value: StringBuilder, cancellationToken: CancellationToken = ...) -> Task:...
        @typing.overload
        def __call__(self, buffer: Array_1[str], index: int, count: int) -> Task:...

    # Skipped WriteLine due to it being static, abstract and generic.

    WriteLine : WriteLine_MethodGroup
    class WriteLine_MethodGroup:
        @typing.overload
        def __call__(self) -> None:...
        @typing.overload
        def __call__(self, value: float) -> None:...
        # Method WriteLine(value : Single) was skipped since it collides with above method
        @typing.overload
        def __call__(self, value: str) -> None:...
        # Method WriteLine(value : Int32) was skipped since it collides with above method
        # Method WriteLine(value : Int64) was skipped since it collides with above method
        # Method WriteLine(value : UInt32) was skipped since it collides with above method
        @typing.overload
        def __call__(self, buffer: Array_1[str]) -> None:...
        # Method WriteLine(s : String) was skipped since it collides with above method
        # Method WriteLine(value : Boolean) was skipped since it collides with above method
        @typing.overload
        def __call__(self, value: typing.Any) -> None:...
        @typing.overload
        def __call__(self, format: str, arg: Array_1[typing.Any]) -> None:...
        @typing.overload
        def __call__(self, format: str, arg0: typing.Any) -> None:...
        @typing.overload
        def __call__(self, buffer: Array_1[str], index: int, count: int) -> None:...
        @typing.overload
        def __call__(self, format: str, arg0: typing.Any, arg1: typing.Any) -> None:...

    # Skipped WriteLineAsync due to it being static, abstract and generic.

    WriteLineAsync : WriteLineAsync_MethodGroup
    class WriteLineAsync_MethodGroup:
        @typing.overload
        def __call__(self) -> Task:...
        @typing.overload
        def __call__(self, value: str) -> Task:...
        # Method WriteLineAsync(value : String) was skipped since it collides with above method
        @typing.overload
        def __call__(self, buffer: ReadOnlyMemory_1[str], cancellationToken: CancellationToken = ...) -> Task:...
        @typing.overload
        def __call__(self, value: StringBuilder, cancellationToken: CancellationToken = ...) -> Task:...
        @typing.overload
        def __call__(self, buffer: Array_1[str], index: int, count: int) -> Task:...





import typing
from System.Reflection import AssemblyName, Assembly
from System.Collections.Generic import IEnumerable_1
from System.IO import Stream
from System import IDisposable

class AssemblyDependencyResolver:
    def __init__(self, componentAssemblyPath: str) -> None: ...
    def ResolveAssemblyToPath(self, assemblyName: AssemblyName) -> str: ...
    def ResolveUnmanagedDllToPath(self, unmanagedDllName: str) -> str: ...


class AssemblyLoadContext:
    def __init__(self, name: str, isCollectible: bool = ...) -> None: ...
    @classmethod
    @property
    def All(cls) -> IEnumerable_1[AssemblyLoadContext]: ...
    @property
    def Assemblies(self) -> IEnumerable_1[Assembly]: ...
    @classmethod
    @property
    def CurrentContextualReflectionContext(cls) -> AssemblyLoadContext: ...
    @classmethod
    @property
    def Default(cls) -> AssemblyLoadContext: ...
    @property
    def IsCollectible(self) -> bool: ...
    @property
    def Name(self) -> str: ...
    @staticmethod
    def GetAssemblyName(assemblyPath: str) -> AssemblyName: ...
    @staticmethod
    def GetLoadContext(assembly: Assembly) -> AssemblyLoadContext: ...
    def LoadFromAssemblyName(self, assemblyName: AssemblyName) -> Assembly: ...
    def LoadFromAssemblyPath(self, assemblyPath: str) -> Assembly: ...
    def LoadFromNativeImagePath(self, nativeImagePath: str, assemblyPath: str) -> Assembly: ...
    def SetProfileOptimizationRoot(self, directoryPath: str) -> None: ...
    def StartProfileOptimization(self, profile: str) -> None: ...
    def ToString(self) -> str: ...
    def Unload(self) -> None: ...
    # Skipped EnterContextualReflection due to it being static, abstract and generic.

    EnterContextualReflection : EnterContextualReflection_MethodGroup
    class EnterContextualReflection_MethodGroup:
        @typing.overload
        def __call__(self) -> AssemblyLoadContext.ContextualReflectionScope:...
        @typing.overload
        def __call__(self, activating: Assembly) -> AssemblyLoadContext.ContextualReflectionScope:...

    # Skipped LoadFromStream due to it being static, abstract and generic.

    LoadFromStream : LoadFromStream_MethodGroup
    class LoadFromStream_MethodGroup:
        @typing.overload
        def __call__(self, assembly: Stream) -> Assembly:...
        @typing.overload
        def __call__(self, assembly: Stream, assemblySymbols: Stream) -> Assembly:...


    class ContextualReflectionScope(IDisposable):
        def Dispose(self) -> None: ...





import typing
from System import IEquatable_1, IFormattable, IFormatProvider, Array_1
from System.ComponentModel import TypeConverter, ITypeDescriptorContext
from System.Globalization import CultureInfo

class SimColor(IEquatable_1[SimColor], IFormattable):
    @property
    def A(self) -> int: ...
    @A.setter
    def A(self, value: int) -> int: ...
    @property
    def B(self) -> int: ...
    @B.setter
    def B(self, value: int) -> int: ...
    @property
    def G(self) -> int: ...
    @G.setter
    def G(self, value: int) -> int: ...
    @property
    def R(self) -> int: ...
    @R.setter
    def R(self, value: int) -> int: ...
    @property
    def ScA(self) -> float: ...
    @ScA.setter
    def ScA(self, value: float) -> float: ...
    @property
    def ScB(self) -> float: ...
    @ScB.setter
    def ScB(self, value: float) -> float: ...
    @property
    def ScG(self) -> float: ...
    @ScG.setter
    def ScG(self, value: float) -> float: ...
    @property
    def ScR(self) -> float: ...
    @ScR.setter
    def ScR(self, value: float) -> float: ...
    @staticmethod
    def Add(color1: SimColor, color2: SimColor) -> SimColor: ...
    def Clamp(self) -> None: ...
    @staticmethod
    def FromArgb(a: int, r: int, g: int, b: int) -> SimColor: ...
    @staticmethod
    def FromRgb(r: int, g: int, b: int) -> SimColor: ...
    @staticmethod
    def FromScRgb(a: float, r: float, g: float, b: float) -> SimColor: ...
    def GetHashCode(self) -> int: ...
    @staticmethod
    def Multiply(color: SimColor, coefficient: float) -> SimColor: ...
    def __add__(self, color1: SimColor, color2: SimColor) -> SimColor: ...
    def __eq__(self, color1: SimColor, color2: SimColor) -> bool: ...
    def __ne__(self, color1: SimColor, color2: SimColor) -> bool: ...
    def __mul__(self, color: SimColor, coefficient: float) -> SimColor: ...
    def __sub__(self, color1: SimColor, color2: SimColor) -> SimColor: ...
    @staticmethod
    def Subtract(color1: SimColor, color2: SimColor) -> SimColor: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, color: SimColor) -> bool:...
        @typing.overload
        def __call__(self, o: typing.Any) -> bool:...
        @typing.overload
        def __call__(self, color1: SimColor, color2: SimColor) -> bool:...

    # Skipped ToString due to it being static, abstract and generic.

    ToString : ToString_MethodGroup
    class ToString_MethodGroup:
        @typing.overload
        def __call__(self) -> str:...
        @typing.overload
        def __call__(self, provider: IFormatProvider) -> str:...



class SimColorConverter(TypeConverter):
    def __init__(self) -> None: ...
    def CanConvertFrom(self, td: ITypeDescriptorContext, t: typing.Type[typing.Any]) -> bool: ...
    def CanConvertTo(self, context: ITypeDescriptorContext, destinationType: typing.Type[typing.Any]) -> bool: ...
    def ConvertFrom(self, td: ITypeDescriptorContext, ci: CultureInfo, value: typing.Any) -> typing.Any: ...
    @staticmethod
    def ConvertFromString(value: str) -> typing.Any: ...
    def ConvertTo(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any, destinationType: typing.Type[typing.Any]) -> typing.Any: ...


class SimColors:
    @classmethod
    @property
    def AliceBlue(cls) -> SimColor: ...
    @classmethod
    @property
    def AntiqueWhite(cls) -> SimColor: ...
    @classmethod
    @property
    def Aqua(cls) -> SimColor: ...
    @classmethod
    @property
    def Aquamarine(cls) -> SimColor: ...
    @classmethod
    @property
    def Azure(cls) -> SimColor: ...
    @classmethod
    @property
    def Beige(cls) -> SimColor: ...
    @classmethod
    @property
    def Bisque(cls) -> SimColor: ...
    @classmethod
    @property
    def Black(cls) -> SimColor: ...
    @classmethod
    @property
    def BlanchedAlmond(cls) -> SimColor: ...
    @classmethod
    @property
    def Blue(cls) -> SimColor: ...
    @classmethod
    @property
    def BlueViolet(cls) -> SimColor: ...
    @classmethod
    @property
    def Brown(cls) -> SimColor: ...
    @classmethod
    @property
    def BurlyWood(cls) -> SimColor: ...
    @classmethod
    @property
    def CadetBlue(cls) -> SimColor: ...
    @classmethod
    @property
    def Chartreuse(cls) -> SimColor: ...
    @classmethod
    @property
    def Chocolate(cls) -> SimColor: ...
    @classmethod
    @property
    def Coral(cls) -> SimColor: ...
    @classmethod
    @property
    def CornflowerBlue(cls) -> SimColor: ...
    @classmethod
    @property
    def Cornsilk(cls) -> SimColor: ...
    @classmethod
    @property
    def Crimson(cls) -> SimColor: ...
    @classmethod
    @property
    def Cyan(cls) -> SimColor: ...
    @classmethod
    @property
    def DarkBlue(cls) -> SimColor: ...
    @classmethod
    @property
    def DarkCyan(cls) -> SimColor: ...
    @classmethod
    @property
    def DarkGoldenrod(cls) -> SimColor: ...
    @classmethod
    @property
    def DarkGray(cls) -> SimColor: ...
    @classmethod
    @property
    def DarkGreen(cls) -> SimColor: ...
    @classmethod
    @property
    def DarkKhaki(cls) -> SimColor: ...
    @classmethod
    @property
    def DarkMagenta(cls) -> SimColor: ...
    @classmethod
    @property
    def DarkOliveGreen(cls) -> SimColor: ...
    @classmethod
    @property
    def DarkOrange(cls) -> SimColor: ...
    @classmethod
    @property
    def DarkOrchid(cls) -> SimColor: ...
    @classmethod
    @property
    def DarkRed(cls) -> SimColor: ...
    @classmethod
    @property
    def DarkSalmon(cls) -> SimColor: ...
    @classmethod
    @property
    def DarkSeaGreen(cls) -> SimColor: ...
    @classmethod
    @property
    def DarkSlateBlue(cls) -> SimColor: ...
    @classmethod
    @property
    def DarkSlateGray(cls) -> SimColor: ...
    @classmethod
    @property
    def DarkTurquoise(cls) -> SimColor: ...
    @classmethod
    @property
    def DarkViolet(cls) -> SimColor: ...
    @classmethod
    @property
    def DeepPink(cls) -> SimColor: ...
    @classmethod
    @property
    def DeepSkyBlue(cls) -> SimColor: ...
    @classmethod
    @property
    def DimGray(cls) -> SimColor: ...
    @classmethod
    @property
    def DodgerBlue(cls) -> SimColor: ...
    @classmethod
    @property
    def Firebrick(cls) -> SimColor: ...
    @classmethod
    @property
    def FloralWhite(cls) -> SimColor: ...
    @classmethod
    @property
    def ForestGreen(cls) -> SimColor: ...
    @classmethod
    @property
    def Fuchsia(cls) -> SimColor: ...
    @classmethod
    @property
    def Gainsboro(cls) -> SimColor: ...
    @classmethod
    @property
    def GhostWhite(cls) -> SimColor: ...
    @classmethod
    @property
    def Gold(cls) -> SimColor: ...
    @classmethod
    @property
    def Goldenrod(cls) -> SimColor: ...
    @classmethod
    @property
    def Gray(cls) -> SimColor: ...
    @classmethod
    @property
    def Green(cls) -> SimColor: ...
    @classmethod
    @property
    def GreenYellow(cls) -> SimColor: ...
    @classmethod
    @property
    def Honeydew(cls) -> SimColor: ...
    @classmethod
    @property
    def HotPink(cls) -> SimColor: ...
    @classmethod
    @property
    def IndianRed(cls) -> SimColor: ...
    @classmethod
    @property
    def Indigo(cls) -> SimColor: ...
    @classmethod
    @property
    def Ivory(cls) -> SimColor: ...
    @classmethod
    @property
    def Khaki(cls) -> SimColor: ...
    @classmethod
    @property
    def Lavender(cls) -> SimColor: ...
    @classmethod
    @property
    def LavenderBlush(cls) -> SimColor: ...
    @classmethod
    @property
    def LawnGreen(cls) -> SimColor: ...
    @classmethod
    @property
    def LemonChiffon(cls) -> SimColor: ...
    @classmethod
    @property
    def LightBlue(cls) -> SimColor: ...
    @classmethod
    @property
    def LightCoral(cls) -> SimColor: ...
    @classmethod
    @property
    def LightCyan(cls) -> SimColor: ...
    @classmethod
    @property
    def LightGoldenrodYellow(cls) -> SimColor: ...
    @classmethod
    @property
    def LightGray(cls) -> SimColor: ...
    @classmethod
    @property
    def LightGreen(cls) -> SimColor: ...
    @classmethod
    @property
    def LightPink(cls) -> SimColor: ...
    @classmethod
    @property
    def LightSalmon(cls) -> SimColor: ...
    @classmethod
    @property
    def LightSeaGreen(cls) -> SimColor: ...
    @classmethod
    @property
    def LightSkyBlue(cls) -> SimColor: ...
    @classmethod
    @property
    def LightSlateGray(cls) -> SimColor: ...
    @classmethod
    @property
    def LightSteelBlue(cls) -> SimColor: ...
    @classmethod
    @property
    def LightYellow(cls) -> SimColor: ...
    @classmethod
    @property
    def Lime(cls) -> SimColor: ...
    @classmethod
    @property
    def LimeGreen(cls) -> SimColor: ...
    @classmethod
    @property
    def Linen(cls) -> SimColor: ...
    @classmethod
    @property
    def Magenta(cls) -> SimColor: ...
    @classmethod
    @property
    def Maroon(cls) -> SimColor: ...
    @classmethod
    @property
    def MediumAquamarine(cls) -> SimColor: ...
    @classmethod
    @property
    def MediumBlue(cls) -> SimColor: ...
    @classmethod
    @property
    def MediumOrchid(cls) -> SimColor: ...
    @classmethod
    @property
    def MediumPurple(cls) -> SimColor: ...
    @classmethod
    @property
    def MediumSeaGreen(cls) -> SimColor: ...
    @classmethod
    @property
    def MediumSlateBlue(cls) -> SimColor: ...
    @classmethod
    @property
    def MediumSpringGreen(cls) -> SimColor: ...
    @classmethod
    @property
    def MediumTurquoise(cls) -> SimColor: ...
    @classmethod
    @property
    def MediumVioletRed(cls) -> SimColor: ...
    @classmethod
    @property
    def MidnightBlue(cls) -> SimColor: ...
    @classmethod
    @property
    def MintCream(cls) -> SimColor: ...
    @classmethod
    @property
    def MistyRose(cls) -> SimColor: ...
    @classmethod
    @property
    def Moccasin(cls) -> SimColor: ...
    @classmethod
    @property
    def NavajoWhite(cls) -> SimColor: ...
    @classmethod
    @property
    def Navy(cls) -> SimColor: ...
    @classmethod
    @property
    def OldLace(cls) -> SimColor: ...
    @classmethod
    @property
    def Olive(cls) -> SimColor: ...
    @classmethod
    @property
    def OliveDrab(cls) -> SimColor: ...
    @classmethod
    @property
    def Orange(cls) -> SimColor: ...
    @classmethod
    @property
    def OrangeRed(cls) -> SimColor: ...
    @classmethod
    @property
    def Orchid(cls) -> SimColor: ...
    @classmethod
    @property
    def PaleGoldenrod(cls) -> SimColor: ...
    @classmethod
    @property
    def PaleGreen(cls) -> SimColor: ...
    @classmethod
    @property
    def PaleTurquoise(cls) -> SimColor: ...
    @classmethod
    @property
    def PaleVioletRed(cls) -> SimColor: ...
    @classmethod
    @property
    def PapayaWhip(cls) -> SimColor: ...
    @classmethod
    @property
    def PeachPuff(cls) -> SimColor: ...
    @classmethod
    @property
    def Peru(cls) -> SimColor: ...
    @classmethod
    @property
    def Pink(cls) -> SimColor: ...
    @classmethod
    @property
    def Plum(cls) -> SimColor: ...
    @classmethod
    @property
    def PowderBlue(cls) -> SimColor: ...
    @classmethod
    @property
    def Purple(cls) -> SimColor: ...
    @classmethod
    @property
    def Red(cls) -> SimColor: ...
    @classmethod
    @property
    def RosyBrown(cls) -> SimColor: ...
    @classmethod
    @property
    def RoyalBlue(cls) -> SimColor: ...
    @classmethod
    @property
    def SaddleBrown(cls) -> SimColor: ...
    @classmethod
    @property
    def Salmon(cls) -> SimColor: ...
    @classmethod
    @property
    def SandyBrown(cls) -> SimColor: ...
    @classmethod
    @property
    def SeaGreen(cls) -> SimColor: ...
    @classmethod
    @property
    def SeaShell(cls) -> SimColor: ...
    @classmethod
    @property
    def Sienna(cls) -> SimColor: ...
    @classmethod
    @property
    def Silver(cls) -> SimColor: ...
    @classmethod
    @property
    def SkyBlue(cls) -> SimColor: ...
    @classmethod
    @property
    def SlateBlue(cls) -> SimColor: ...
    @classmethod
    @property
    def SlateGray(cls) -> SimColor: ...
    @classmethod
    @property
    def Snow(cls) -> SimColor: ...
    @classmethod
    @property
    def SpringGreen(cls) -> SimColor: ...
    @classmethod
    @property
    def SteelBlue(cls) -> SimColor: ...
    @classmethod
    @property
    def Tan(cls) -> SimColor: ...
    @classmethod
    @property
    def Teal(cls) -> SimColor: ...
    @classmethod
    @property
    def Thistle(cls) -> SimColor: ...
    @classmethod
    @property
    def Tomato(cls) -> SimColor: ...
    @classmethod
    @property
    def Transparent(cls) -> SimColor: ...
    @classmethod
    @property
    def Turquoise(cls) -> SimColor: ...
    @classmethod
    @property
    def Violet(cls) -> SimColor: ...
    @classmethod
    @property
    def Wheat(cls) -> SimColor: ...
    @classmethod
    @property
    def White(cls) -> SimColor: ...
    @classmethod
    @property
    def WhiteSmoke(cls) -> SimColor: ...
    @classmethod
    @property
    def Yellow(cls) -> SimColor: ...
    @classmethod
    @property
    def YellowGreen(cls) -> SimColor: ...


class SimMatrix(IFormattable):
    def __init__(self, m11: float, m12: float, m21: float, m22: float, offsetX: float, offsetY: float) -> None: ...
    @property
    def Determinant(self) -> float: ...
    @property
    def HasInverse(self) -> bool: ...
    @classmethod
    @property
    def Identity(cls) -> SimMatrix: ...
    @property
    def IsIdentity(self) -> bool: ...
    @property
    def M11(self) -> float: ...
    @M11.setter
    def M11(self, value: float) -> float: ...
    @property
    def M12(self) -> float: ...
    @M12.setter
    def M12(self, value: float) -> float: ...
    @property
    def M21(self) -> float: ...
    @M21.setter
    def M21(self, value: float) -> float: ...
    @property
    def M22(self) -> float: ...
    @M22.setter
    def M22(self, value: float) -> float: ...
    @property
    def OffsetX(self) -> float: ...
    @OffsetX.setter
    def OffsetX(self, value: float) -> float: ...
    @property
    def OffsetY(self) -> float: ...
    @OffsetY.setter
    def OffsetY(self, value: float) -> float: ...
    def Append(self, matrix: SimMatrix) -> None: ...
    def GetHashCode(self) -> int: ...
    def Invert(self) -> None: ...
    @staticmethod
    def Multiply(trans1: SimMatrix, trans2: SimMatrix) -> SimMatrix: ...
    def __eq__(self, matrix1: SimMatrix, matrix2: SimMatrix) -> bool: ...
    def __ne__(self, matrix1: SimMatrix, matrix2: SimMatrix) -> bool: ...
    def __mul__(self, trans1: SimMatrix, trans2: SimMatrix) -> SimMatrix: ...
    @staticmethod
    def Parse(source: str) -> SimMatrix: ...
    def Prepend(self, matrix: SimMatrix) -> None: ...
    def Rotate(self, angle: float) -> None: ...
    def RotateAt(self, angle: float, centerX: float, centerY: float) -> None: ...
    def RotateAtPrepend(self, angle: float, centerX: float, centerY: float) -> None: ...
    def RotatePrepend(self, angle: float) -> None: ...
    def Scale(self, scaleX: float, scaleY: float) -> None: ...
    def ScaleAt(self, scaleX: float, scaleY: float, centerX: float, centerY: float) -> None: ...
    def ScaleAtPrepend(self, scaleX: float, scaleY: float, centerX: float, centerY: float) -> None: ...
    def ScalePrepend(self, scaleX: float, scaleY: float) -> None: ...
    def SetIdentity(self) -> None: ...
    def Skew(self, skewX: float, skewY: float) -> None: ...
    def SkewPrepend(self, skewX: float, skewY: float) -> None: ...
    def Translate(self, offsetX: float, offsetY: float) -> None: ...
    def TranslatePrepend(self, offsetX: float, offsetY: float) -> None: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, value: SimMatrix) -> bool:...
        @typing.overload
        def __call__(self, o: typing.Any) -> bool:...
        @typing.overload
        def __call__(self, matrix1: SimMatrix, matrix2: SimMatrix) -> bool:...

    # Skipped ToString due to it being static, abstract and generic.

    ToString : ToString_MethodGroup
    class ToString_MethodGroup:
        @typing.overload
        def __call__(self) -> str:...
        @typing.overload
        def __call__(self, provider: IFormatProvider) -> str:...

    # Skipped Transform due to it being static, abstract and generic.

    Transform : Transform_MethodGroup
    class Transform_MethodGroup:
        @typing.overload
        def __call__(self, points: Array_1[SimPoint]) -> None:...
        @typing.overload
        def __call__(self, vectors: Array_1[SimVector]) -> None:...
        @typing.overload
        def __call__(self, point: SimPoint) -> SimPoint:...
        @typing.overload
        def __call__(self, vector: SimVector) -> SimVector:...



class SimMatrix3D(IFormattable):
    def __init__(self, m11: float, m12: float, m13: float, m14: float, m21: float, m22: float, m23: float, m24: float, m31: float, m32: float, m33: float, m34: float, offsetX: float, offsetY: float, offsetZ: float, m44: float) -> None: ...
    @property
    def Determinant(self) -> float: ...
    @property
    def HasInverse(self) -> bool: ...
    @classmethod
    @property
    def Identity(cls) -> SimMatrix3D: ...
    @property
    def IsAffine(self) -> bool: ...
    @property
    def IsIdentity(self) -> bool: ...
    @property
    def M11(self) -> float: ...
    @M11.setter
    def M11(self, value: float) -> float: ...
    @property
    def M12(self) -> float: ...
    @M12.setter
    def M12(self, value: float) -> float: ...
    @property
    def M13(self) -> float: ...
    @M13.setter
    def M13(self, value: float) -> float: ...
    @property
    def M14(self) -> float: ...
    @M14.setter
    def M14(self, value: float) -> float: ...
    @property
    def M21(self) -> float: ...
    @M21.setter
    def M21(self, value: float) -> float: ...
    @property
    def M22(self) -> float: ...
    @M22.setter
    def M22(self, value: float) -> float: ...
    @property
    def M23(self) -> float: ...
    @M23.setter
    def M23(self, value: float) -> float: ...
    @property
    def M24(self) -> float: ...
    @M24.setter
    def M24(self, value: float) -> float: ...
    @property
    def M31(self) -> float: ...
    @M31.setter
    def M31(self, value: float) -> float: ...
    @property
    def M32(self) -> float: ...
    @M32.setter
    def M32(self, value: float) -> float: ...
    @property
    def M33(self) -> float: ...
    @M33.setter
    def M33(self, value: float) -> float: ...
    @property
    def M34(self) -> float: ...
    @M34.setter
    def M34(self, value: float) -> float: ...
    @property
    def M44(self) -> float: ...
    @M44.setter
    def M44(self, value: float) -> float: ...
    @property
    def OffsetX(self) -> float: ...
    @OffsetX.setter
    def OffsetX(self, value: float) -> float: ...
    @property
    def OffsetY(self) -> float: ...
    @OffsetY.setter
    def OffsetY(self, value: float) -> float: ...
    @property
    def OffsetZ(self) -> float: ...
    @OffsetZ.setter
    def OffsetZ(self, value: float) -> float: ...
    def Append(self, matrix: SimMatrix3D) -> None: ...
    def GetHashCode(self) -> int: ...
    def Invert(self) -> None: ...
    @staticmethod
    def Multiply(matrix1: SimMatrix3D, matrix2: SimMatrix3D) -> SimMatrix3D: ...
    def __eq__(self, matrix1: SimMatrix3D, matrix2: SimMatrix3D) -> bool: ...
    def __ne__(self, matrix1: SimMatrix3D, matrix2: SimMatrix3D) -> bool: ...
    def __mul__(self, matrix1: SimMatrix3D, matrix2: SimMatrix3D) -> SimMatrix3D: ...
    @staticmethod
    def Parse(source: str) -> SimMatrix3D: ...
    def Prepend(self, matrix: SimMatrix3D) -> None: ...
    def Rotate(self, quaternion: SimQuaternion) -> None: ...
    def RotateAt(self, quaternion: SimQuaternion, center: SimPoint3D) -> None: ...
    def RotateAtPrepend(self, quaternion: SimQuaternion, center: SimPoint3D) -> None: ...
    def RotatePrepend(self, quaternion: SimQuaternion) -> None: ...
    def Scale(self, scale: SimVector3D) -> None: ...
    def ScaleAt(self, scale: SimVector3D, center: SimPoint3D) -> None: ...
    def ScaleAtPrepend(self, scale: SimVector3D, center: SimPoint3D) -> None: ...
    def ScalePrepend(self, scale: SimVector3D) -> None: ...
    def SetIdentity(self) -> None: ...
    def Translate(self, offset: SimVector3D) -> None: ...
    def TranslatePrepend(self, offset: SimVector3D) -> None: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, value: SimMatrix3D) -> bool:...
        @typing.overload
        def __call__(self, o: typing.Any) -> bool:...
        @typing.overload
        def __call__(self, matrix1: SimMatrix3D, matrix2: SimMatrix3D) -> bool:...

    # Skipped ToString due to it being static, abstract and generic.

    ToString : ToString_MethodGroup
    class ToString_MethodGroup:
        @typing.overload
        def __call__(self) -> str:...
        @typing.overload
        def __call__(self, provider: IFormatProvider) -> str:...

    # Skipped Transform due to it being static, abstract and generic.

    Transform : Transform_MethodGroup
    class Transform_MethodGroup:
        @typing.overload
        def __call__(self, points: Array_1[SimPoint3D]) -> None:...
        @typing.overload
        def __call__(self, points: Array_1[SimPoint4D]) -> None:...
        @typing.overload
        def __call__(self, vectors: Array_1[SimVector3D]) -> None:...
        @typing.overload
        def __call__(self, point: SimPoint3D) -> SimPoint3D:...
        @typing.overload
        def __call__(self, point: SimPoint4D) -> SimPoint4D:...
        @typing.overload
        def __call__(self, vector: SimVector3D) -> SimVector3D:...



class SimMatrix3DConverter(TypeConverter):
    def __init__(self) -> None: ...
    def CanConvertFrom(self, context: ITypeDescriptorContext, sourceType: typing.Type[typing.Any]) -> bool: ...
    def CanConvertTo(self, context: ITypeDescriptorContext, destinationType: typing.Type[typing.Any]) -> bool: ...
    def ConvertFrom(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any) -> typing.Any: ...
    def ConvertTo(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any, destinationType: typing.Type[typing.Any]) -> typing.Any: ...


class SimMatrixConverter(TypeConverter):
    def __init__(self) -> None: ...
    def CanConvertFrom(self, context: ITypeDescriptorContext, sourceType: typing.Type[typing.Any]) -> bool: ...
    def CanConvertTo(self, context: ITypeDescriptorContext, destinationType: typing.Type[typing.Any]) -> bool: ...
    def ConvertFrom(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any) -> typing.Any: ...
    def ConvertTo(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any, destinationType: typing.Type[typing.Any]) -> typing.Any: ...


class SimPoint(IFormattable):
    def __init__(self, x: float, y: float) -> None: ...
    @property
    def X(self) -> float: ...
    @X.setter
    def X(self, value: float) -> float: ...
    @property
    def Y(self) -> float: ...
    @Y.setter
    def Y(self, value: float) -> float: ...
    @staticmethod
    def Add(point: SimPoint, vector: SimVector) -> SimPoint: ...
    def GetHashCode(self) -> int: ...
    @staticmethod
    def Multiply(point: SimPoint, matrix: SimMatrix) -> SimPoint: ...
    def Offset(self, offsetX: float, offsetY: float) -> None: ...
    def __add__(self, point: SimPoint, vector: SimVector) -> SimPoint: ...
    def __eq__(self, point1: SimPoint, point2: SimPoint) -> bool: ...
    # Operator not supported op_Explicit(point: SimPoint)
    # Operator not supported op_Explicit(point: SimPoint)
    def __ne__(self, point1: SimPoint, point2: SimPoint) -> bool: ...
    def __mul__(self, point: SimPoint, matrix: SimMatrix) -> SimPoint: ...
    @typing.overload
    def __sub__(self, point1: SimPoint, point2: SimPoint) -> SimVector: ...
    @typing.overload
    def __sub__(self, point: SimPoint, vector: SimVector) -> SimPoint: ...
    @staticmethod
    def Parse(source: str) -> SimPoint: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, value: SimPoint) -> bool:...
        @typing.overload
        def __call__(self, o: typing.Any) -> bool:...
        @typing.overload
        def __call__(self, point1: SimPoint, point2: SimPoint) -> bool:...

    # Skipped Subtract due to it being static, abstract and generic.

    Subtract : Subtract_MethodGroup
    class Subtract_MethodGroup:
        @typing.overload
        def __call__(self, point1: SimPoint, point2: SimPoint) -> SimVector:...
        @typing.overload
        def __call__(self, point: SimPoint, vector: SimVector) -> SimPoint:...

    # Skipped ToString due to it being static, abstract and generic.

    ToString : ToString_MethodGroup
    class ToString_MethodGroup:
        @typing.overload
        def __call__(self) -> str:...
        @typing.overload
        def __call__(self, provider: IFormatProvider) -> str:...



class SimPoint3D(IFormattable):
    def __init__(self, x: float, y: float, z: float) -> None: ...
    @property
    def X(self) -> float: ...
    @X.setter
    def X(self, value: float) -> float: ...
    @property
    def Y(self) -> float: ...
    @Y.setter
    def Y(self, value: float) -> float: ...
    @property
    def Z(self) -> float: ...
    @Z.setter
    def Z(self, value: float) -> float: ...
    @staticmethod
    def Add(point: SimPoint3D, vector: SimVector3D) -> SimPoint3D: ...
    def GetHashCode(self) -> int: ...
    @staticmethod
    def Multiply(point: SimPoint3D, matrix: SimMatrix3D) -> SimPoint3D: ...
    def Offset(self, offsetX: float, offsetY: float, offsetZ: float) -> None: ...
    def __add__(self, point: SimPoint3D, vector: SimVector3D) -> SimPoint3D: ...
    def __eq__(self, point1: SimPoint3D, point2: SimPoint3D) -> bool: ...
    # Operator not supported op_Explicit(point: SimPoint3D)
    # Operator not supported op_Explicit(point: SimPoint3D)
    def __ne__(self, point1: SimPoint3D, point2: SimPoint3D) -> bool: ...
    def __mul__(self, point: SimPoint3D, matrix: SimMatrix3D) -> SimPoint3D: ...
    @typing.overload
    def __sub__(self, point1: SimPoint3D, point2: SimPoint3D) -> SimVector3D: ...
    @typing.overload
    def __sub__(self, point: SimPoint3D, vector: SimVector3D) -> SimPoint3D: ...
    @staticmethod
    def Parse(source: str) -> SimPoint3D: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, value: SimPoint3D) -> bool:...
        @typing.overload
        def __call__(self, o: typing.Any) -> bool:...
        @typing.overload
        def __call__(self, point1: SimPoint3D, point2: SimPoint3D) -> bool:...

    # Skipped Subtract due to it being static, abstract and generic.

    Subtract : Subtract_MethodGroup
    class Subtract_MethodGroup:
        @typing.overload
        def __call__(self, point1: SimPoint3D, point2: SimPoint3D) -> SimVector3D:...
        @typing.overload
        def __call__(self, point: SimPoint3D, vector: SimVector3D) -> SimPoint3D:...

    # Skipped ToString due to it being static, abstract and generic.

    ToString : ToString_MethodGroup
    class ToString_MethodGroup:
        @typing.overload
        def __call__(self) -> str:...
        @typing.overload
        def __call__(self, provider: IFormatProvider) -> str:...



class SimPoint3DConverter(TypeConverter):
    def __init__(self) -> None: ...
    def CanConvertFrom(self, context: ITypeDescriptorContext, sourceType: typing.Type[typing.Any]) -> bool: ...
    def CanConvertTo(self, context: ITypeDescriptorContext, destinationType: typing.Type[typing.Any]) -> bool: ...
    def ConvertFrom(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any) -> typing.Any: ...
    def ConvertTo(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any, destinationType: typing.Type[typing.Any]) -> typing.Any: ...


class SimPoint4D(IFormattable):
    def __init__(self, x: float, y: float, z: float, w: float) -> None: ...
    @property
    def W(self) -> float: ...
    @W.setter
    def W(self, value: float) -> float: ...
    @property
    def X(self) -> float: ...
    @X.setter
    def X(self, value: float) -> float: ...
    @property
    def Y(self) -> float: ...
    @Y.setter
    def Y(self, value: float) -> float: ...
    @property
    def Z(self) -> float: ...
    @Z.setter
    def Z(self, value: float) -> float: ...
    @staticmethod
    def Add(point1: SimPoint4D, point2: SimPoint4D) -> SimPoint4D: ...
    def GetHashCode(self) -> int: ...
    @staticmethod
    def Multiply(point: SimPoint4D, matrix: SimMatrix3D) -> SimPoint4D: ...
    def Offset(self, deltaX: float, deltaY: float, deltaZ: float, deltaW: float) -> None: ...
    def __add__(self, point1: SimPoint4D, point2: SimPoint4D) -> SimPoint4D: ...
    def __eq__(self, point1: SimPoint4D, point2: SimPoint4D) -> bool: ...
    def __ne__(self, point1: SimPoint4D, point2: SimPoint4D) -> bool: ...
    def __mul__(self, point: SimPoint4D, matrix: SimMatrix3D) -> SimPoint4D: ...
    def __sub__(self, point1: SimPoint4D, point2: SimPoint4D) -> SimPoint4D: ...
    @staticmethod
    def Parse(source: str) -> SimPoint4D: ...
    @staticmethod
    def Subtract(point1: SimPoint4D, point2: SimPoint4D) -> SimPoint4D: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, value: SimPoint4D) -> bool:...
        @typing.overload
        def __call__(self, o: typing.Any) -> bool:...
        @typing.overload
        def __call__(self, point1: SimPoint4D, point2: SimPoint4D) -> bool:...

    # Skipped ToString due to it being static, abstract and generic.

    ToString : ToString_MethodGroup
    class ToString_MethodGroup:
        @typing.overload
        def __call__(self) -> str:...
        @typing.overload
        def __call__(self, provider: IFormatProvider) -> str:...



class SimPoint4DConverter(TypeConverter):
    def __init__(self) -> None: ...
    def CanConvertFrom(self, context: ITypeDescriptorContext, sourceType: typing.Type[typing.Any]) -> bool: ...
    def CanConvertTo(self, context: ITypeDescriptorContext, destinationType: typing.Type[typing.Any]) -> bool: ...
    def ConvertFrom(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any) -> typing.Any: ...
    def ConvertTo(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any, destinationType: typing.Type[typing.Any]) -> typing.Any: ...


class SimPointConverter(TypeConverter):
    def __init__(self) -> None: ...
    def CanConvertFrom(self, context: ITypeDescriptorContext, sourceType: typing.Type[typing.Any]) -> bool: ...
    def CanConvertTo(self, context: ITypeDescriptorContext, destinationType: typing.Type[typing.Any]) -> bool: ...
    def ConvertFrom(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any) -> typing.Any: ...
    def ConvertTo(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any, destinationType: typing.Type[typing.Any]) -> typing.Any: ...


class SimQuaternion(IFormattable):
    @typing.overload
    def __init__(self, axisOfRotation: SimVector3D, angleInDegrees: float) -> None: ...
    @typing.overload
    def __init__(self, x: float, y: float, z: float, w: float) -> None: ...
    @property
    def Angle(self) -> float: ...
    @property
    def Axis(self) -> SimVector3D: ...
    @classmethod
    @property
    def Identity(cls) -> SimQuaternion: ...
    @property
    def IsIdentity(self) -> bool: ...
    @property
    def IsNormalized(self) -> bool: ...
    @property
    def W(self) -> float: ...
    @W.setter
    def W(self, value: float) -> float: ...
    @property
    def X(self) -> float: ...
    @X.setter
    def X(self, value: float) -> float: ...
    @property
    def Y(self) -> float: ...
    @Y.setter
    def Y(self, value: float) -> float: ...
    @property
    def Z(self) -> float: ...
    @Z.setter
    def Z(self, value: float) -> float: ...
    @staticmethod
    def Add(left: SimQuaternion, right: SimQuaternion) -> SimQuaternion: ...
    def Conjugate(self) -> None: ...
    def GetHashCode(self) -> int: ...
    def Invert(self) -> None: ...
    @staticmethod
    def Multiply(left: SimQuaternion, right: SimQuaternion) -> SimQuaternion: ...
    def Normalize(self) -> None: ...
    def __add__(self, left: SimQuaternion, right: SimQuaternion) -> SimQuaternion: ...
    def __eq__(self, quaternion1: SimQuaternion, quaternion2: SimQuaternion) -> bool: ...
    def __ne__(self, quaternion1: SimQuaternion, quaternion2: SimQuaternion) -> bool: ...
    def __mul__(self, left: SimQuaternion, right: SimQuaternion) -> SimQuaternion: ...
    def __sub__(self, left: SimQuaternion, right: SimQuaternion) -> SimQuaternion: ...
    @staticmethod
    def Parse(source: str) -> SimQuaternion: ...
    @staticmethod
    def Subtract(left: SimQuaternion, right: SimQuaternion) -> SimQuaternion: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, value: SimQuaternion) -> bool:...
        @typing.overload
        def __call__(self, o: typing.Any) -> bool:...
        @typing.overload
        def __call__(self, quaternion1: SimQuaternion, quaternion2: SimQuaternion) -> bool:...

    # Skipped Slerp due to it being static, abstract and generic.

    Slerp : Slerp_MethodGroup
    class Slerp_MethodGroup:
        @typing.overload
        def __call__(self, from_: SimQuaternion, to: SimQuaternion, t: float) -> SimQuaternion:...
        @typing.overload
        def __call__(self, from_: SimQuaternion, to: SimQuaternion, t: float, useShortestPath: bool) -> SimQuaternion:...

    # Skipped ToString due to it being static, abstract and generic.

    ToString : ToString_MethodGroup
    class ToString_MethodGroup:
        @typing.overload
        def __call__(self) -> str:...
        @typing.overload
        def __call__(self, provider: IFormatProvider) -> str:...



class SimQuaternionConverter(TypeConverter):
    def __init__(self) -> None: ...
    def CanConvertFrom(self, context: ITypeDescriptorContext, sourceType: typing.Type[typing.Any]) -> bool: ...
    def CanConvertTo(self, context: ITypeDescriptorContext, destinationType: typing.Type[typing.Any]) -> bool: ...
    def ConvertFrom(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any) -> typing.Any: ...
    def ConvertTo(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any, destinationType: typing.Type[typing.Any]) -> typing.Any: ...


class SimRect(IFormattable):
    @typing.overload
    def __init__(self, location: SimPoint, size: SimSize) -> None: ...
    @typing.overload
    def __init__(self, point: SimPoint, vector: SimVector) -> None: ...
    @typing.overload
    def __init__(self, point1: SimPoint, point2: SimPoint) -> None: ...
    @typing.overload
    def __init__(self, size: SimSize) -> None: ...
    @typing.overload
    def __init__(self, x: float, y: float, width: float, height: float) -> None: ...
    @property
    def Bottom(self) -> float: ...
    @property
    def BottomLeft(self) -> SimPoint: ...
    @property
    def BottomRight(self) -> SimPoint: ...
    @classmethod
    @property
    def Empty(cls) -> SimRect: ...
    @property
    def Height(self) -> float: ...
    @Height.setter
    def Height(self, value: float) -> float: ...
    @property
    def IsEmpty(self) -> bool: ...
    @property
    def Left(self) -> float: ...
    @property
    def Location(self) -> SimPoint: ...
    @Location.setter
    def Location(self, value: SimPoint) -> SimPoint: ...
    @property
    def Right(self) -> float: ...
    @property
    def Size(self) -> SimSize: ...
    @Size.setter
    def Size(self, value: SimSize) -> SimSize: ...
    @property
    def Top(self) -> float: ...
    @property
    def TopLeft(self) -> SimPoint: ...
    @property
    def TopRight(self) -> SimPoint: ...
    @property
    def Width(self) -> float: ...
    @Width.setter
    def Width(self, value: float) -> float: ...
    @property
    def X(self) -> float: ...
    @X.setter
    def X(self, value: float) -> float: ...
    @property
    def Y(self) -> float: ...
    @Y.setter
    def Y(self, value: float) -> float: ...
    def GetHashCode(self) -> int: ...
    def IntersectsWith(self, rect: SimRect) -> bool: ...
    def __eq__(self, rect1: SimRect, rect2: SimRect) -> bool: ...
    def __ne__(self, rect1: SimRect, rect2: SimRect) -> bool: ...
    @staticmethod
    def Parse(source: str) -> SimRect: ...
    def Scale(self, scaleX: float, scaleY: float) -> None: ...
    # Skipped Contains due to it being static, abstract and generic.

    Contains : Contains_MethodGroup
    class Contains_MethodGroup:
        @typing.overload
        def __call__(self, point: SimPoint) -> bool:...
        @typing.overload
        def __call__(self, rect: SimRect) -> bool:...
        @typing.overload
        def __call__(self, x: float, y: float) -> bool:...

    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, value: SimRect) -> bool:...
        @typing.overload
        def __call__(self, o: typing.Any) -> bool:...
        @typing.overload
        def __call__(self, rect1: SimRect, rect2: SimRect) -> bool:...

    # Skipped Inflate due to it being static, abstract and generic.

    Inflate : Inflate_MethodGroup
    class Inflate_MethodGroup:
        @typing.overload
        def __call__(self, size: SimSize) -> None:...
        @typing.overload
        def __call__(self, width: float, height: float) -> None:...
        @typing.overload
        def __call__(self, rect: SimRect, size: SimSize) -> SimRect:...
        @typing.overload
        def __call__(self, rect: SimRect, width: float, height: float) -> SimRect:...

    # Skipped Intersect due to it being static, abstract and generic.

    Intersect : Intersect_MethodGroup
    class Intersect_MethodGroup:
        @typing.overload
        def __call__(self, rect: SimRect) -> None:...
        @typing.overload
        def __call__(self, rect1: SimRect, rect2: SimRect) -> SimRect:...

    # Skipped Offset due to it being static, abstract and generic.

    Offset : Offset_MethodGroup
    class Offset_MethodGroup:
        @typing.overload
        def __call__(self, offsetVector: SimVector) -> None:...
        @typing.overload
        def __call__(self, offsetX: float, offsetY: float) -> None:...
        @typing.overload
        def __call__(self, rect: SimRect, offsetVector: SimVector) -> SimRect:...
        @typing.overload
        def __call__(self, rect: SimRect, offsetX: float, offsetY: float) -> SimRect:...

    # Skipped ToString due to it being static, abstract and generic.

    ToString : ToString_MethodGroup
    class ToString_MethodGroup:
        @typing.overload
        def __call__(self) -> str:...
        @typing.overload
        def __call__(self, provider: IFormatProvider) -> str:...

    # Skipped Transform due to it being static, abstract and generic.

    Transform : Transform_MethodGroup
    class Transform_MethodGroup:
        @typing.overload
        def __call__(self, matrix: SimMatrix) -> None:...
        @typing.overload
        def __call__(self, rect: SimRect, matrix: SimMatrix) -> SimRect:...

    # Skipped Union due to it being static, abstract and generic.

    Union : Union_MethodGroup
    class Union_MethodGroup:
        @typing.overload
        def __call__(self, point: SimPoint) -> None:...
        @typing.overload
        def __call__(self, rect: SimRect) -> None:...
        @typing.overload
        def __call__(self, rect1: SimRect, rect2: SimRect) -> SimRect:...
        @typing.overload
        def __call__(self, rect: SimRect, point: SimPoint) -> SimRect:...



class SimRect3D(IFormattable):
    @typing.overload
    def __init__(self, location: SimPoint3D, size: SimSize3D) -> None: ...
    @typing.overload
    def __init__(self, x: float, y: float, z: float, sizeX: float, sizeY: float, sizeZ: float) -> None: ...
    @classmethod
    @property
    def Empty(cls) -> SimRect3D: ...
    @property
    def IsEmpty(self) -> bool: ...
    @property
    def Location(self) -> SimPoint3D: ...
    @Location.setter
    def Location(self, value: SimPoint3D) -> SimPoint3D: ...
    @property
    def Size(self) -> SimSize3D: ...
    @Size.setter
    def Size(self, value: SimSize3D) -> SimSize3D: ...
    @property
    def SizeX(self) -> float: ...
    @SizeX.setter
    def SizeX(self, value: float) -> float: ...
    @property
    def SizeY(self) -> float: ...
    @SizeY.setter
    def SizeY(self, value: float) -> float: ...
    @property
    def SizeZ(self) -> float: ...
    @SizeZ.setter
    def SizeZ(self, value: float) -> float: ...
    @property
    def X(self) -> float: ...
    @X.setter
    def X(self, value: float) -> float: ...
    @property
    def Y(self) -> float: ...
    @Y.setter
    def Y(self, value: float) -> float: ...
    @property
    def Z(self) -> float: ...
    @Z.setter
    def Z(self, value: float) -> float: ...
    def GetHashCode(self) -> int: ...
    def IntersectsWith(self, rect: SimRect3D) -> bool: ...
    def __eq__(self, rect1: SimRect3D, rect2: SimRect3D) -> bool: ...
    def __ne__(self, rect1: SimRect3D, rect2: SimRect3D) -> bool: ...
    @staticmethod
    def Parse(source: str) -> SimRect3D: ...
    # Skipped Contains due to it being static, abstract and generic.

    Contains : Contains_MethodGroup
    class Contains_MethodGroup:
        @typing.overload
        def __call__(self, point: SimPoint3D) -> bool:...
        @typing.overload
        def __call__(self, rect: SimRect3D) -> bool:...
        @typing.overload
        def __call__(self, x: float, y: float, z: float) -> bool:...

    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, value: SimRect3D) -> bool:...
        @typing.overload
        def __call__(self, o: typing.Any) -> bool:...
        @typing.overload
        def __call__(self, rect1: SimRect3D, rect2: SimRect3D) -> bool:...

    # Skipped Intersect due to it being static, abstract and generic.

    Intersect : Intersect_MethodGroup
    class Intersect_MethodGroup:
        @typing.overload
        def __call__(self, rect: SimRect3D) -> None:...
        @typing.overload
        def __call__(self, rect1: SimRect3D, rect2: SimRect3D) -> SimRect3D:...

    # Skipped Offset due to it being static, abstract and generic.

    Offset : Offset_MethodGroup
    class Offset_MethodGroup:
        @typing.overload
        def __call__(self, offsetVector: SimVector3D) -> None:...
        @typing.overload
        def __call__(self, rect: SimRect3D, offsetVector: SimVector3D) -> SimRect3D:...
        @typing.overload
        def __call__(self, offsetX: float, offsetY: float, offsetZ: float) -> None:...
        @typing.overload
        def __call__(self, rect: SimRect3D, offsetX: float, offsetY: float, offsetZ: float) -> SimRect3D:...

    # Skipped ToString due to it being static, abstract and generic.

    ToString : ToString_MethodGroup
    class ToString_MethodGroup:
        @typing.overload
        def __call__(self) -> str:...
        @typing.overload
        def __call__(self, provider: IFormatProvider) -> str:...

    # Skipped Union due to it being static, abstract and generic.

    Union : Union_MethodGroup
    class Union_MethodGroup:
        @typing.overload
        def __call__(self, point: SimPoint3D) -> None:...
        @typing.overload
        def __call__(self, rect: SimRect3D) -> None:...
        @typing.overload
        def __call__(self, rect1: SimRect3D, rect2: SimRect3D) -> SimRect3D:...
        @typing.overload
        def __call__(self, rect: SimRect3D, point: SimPoint3D) -> SimRect3D:...



class SimRect3DConverter(TypeConverter):
    def __init__(self) -> None: ...
    def CanConvertFrom(self, context: ITypeDescriptorContext, sourceType: typing.Type[typing.Any]) -> bool: ...
    def CanConvertTo(self, context: ITypeDescriptorContext, destinationType: typing.Type[typing.Any]) -> bool: ...
    def ConvertFrom(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any) -> typing.Any: ...
    def ConvertTo(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any, destinationType: typing.Type[typing.Any]) -> typing.Any: ...


class SimRectConverter(TypeConverter):
    def __init__(self) -> None: ...
    def CanConvertFrom(self, context: ITypeDescriptorContext, sourceType: typing.Type[typing.Any]) -> bool: ...
    def CanConvertTo(self, context: ITypeDescriptorContext, destinationType: typing.Type[typing.Any]) -> bool: ...
    def ConvertFrom(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any) -> typing.Any: ...
    def ConvertTo(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any, destinationType: typing.Type[typing.Any]) -> typing.Any: ...


class SimSize(IFormattable):
    def __init__(self, width: float, height: float) -> None: ...
    @classmethod
    @property
    def Empty(cls) -> SimSize: ...
    @property
    def Height(self) -> float: ...
    @Height.setter
    def Height(self, value: float) -> float: ...
    @property
    def IsEmpty(self) -> bool: ...
    @property
    def Width(self) -> float: ...
    @Width.setter
    def Width(self, value: float) -> float: ...
    def GetHashCode(self) -> int: ...
    def __eq__(self, size1: SimSize, size2: SimSize) -> bool: ...
    # Operator not supported op_Explicit(size: SimSize)
    # Operator not supported op_Explicit(size: SimSize)
    def __ne__(self, size1: SimSize, size2: SimSize) -> bool: ...
    @staticmethod
    def Parse(source: str) -> SimSize: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, value: SimSize) -> bool:...
        @typing.overload
        def __call__(self, o: typing.Any) -> bool:...
        @typing.overload
        def __call__(self, size1: SimSize, size2: SimSize) -> bool:...

    # Skipped ToString due to it being static, abstract and generic.

    ToString : ToString_MethodGroup
    class ToString_MethodGroup:
        @typing.overload
        def __call__(self) -> str:...
        @typing.overload
        def __call__(self, provider: IFormatProvider) -> str:...



class SimSize3D(IFormattable):
    def __init__(self, x: float, y: float, z: float) -> None: ...
    @classmethod
    @property
    def Empty(cls) -> SimSize3D: ...
    @property
    def IsEmpty(self) -> bool: ...
    @property
    def X(self) -> float: ...
    @X.setter
    def X(self, value: float) -> float: ...
    @property
    def Y(self) -> float: ...
    @Y.setter
    def Y(self, value: float) -> float: ...
    @property
    def Z(self) -> float: ...
    @Z.setter
    def Z(self, value: float) -> float: ...
    def GetHashCode(self) -> int: ...
    def __eq__(self, size1: SimSize3D, size2: SimSize3D) -> bool: ...
    # Operator not supported op_Explicit(size: SimSize3D)
    # Operator not supported op_Explicit(size: SimSize3D)
    def __ne__(self, size1: SimSize3D, size2: SimSize3D) -> bool: ...
    @staticmethod
    def Parse(source: str) -> SimSize3D: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, value: SimSize3D) -> bool:...
        @typing.overload
        def __call__(self, o: typing.Any) -> bool:...
        @typing.overload
        def __call__(self, size1: SimSize3D, size2: SimSize3D) -> bool:...

    # Skipped ToString due to it being static, abstract and generic.

    ToString : ToString_MethodGroup
    class ToString_MethodGroup:
        @typing.overload
        def __call__(self) -> str:...
        @typing.overload
        def __call__(self, provider: IFormatProvider) -> str:...



class SimSize3DConverter(TypeConverter):
    def __init__(self) -> None: ...
    def CanConvertFrom(self, context: ITypeDescriptorContext, sourceType: typing.Type[typing.Any]) -> bool: ...
    def CanConvertTo(self, context: ITypeDescriptorContext, destinationType: typing.Type[typing.Any]) -> bool: ...
    def ConvertFrom(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any) -> typing.Any: ...
    def ConvertTo(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any, destinationType: typing.Type[typing.Any]) -> typing.Any: ...


class SimSizeConverter(TypeConverter):
    def __init__(self) -> None: ...
    def CanConvertFrom(self, context: ITypeDescriptorContext, sourceType: typing.Type[typing.Any]) -> bool: ...
    def CanConvertTo(self, context: ITypeDescriptorContext, destinationType: typing.Type[typing.Any]) -> bool: ...
    def ConvertFrom(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any) -> typing.Any: ...
    def ConvertTo(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any, destinationType: typing.Type[typing.Any]) -> typing.Any: ...


class SimVector(IFormattable):
    def __init__(self, x: float, y: float) -> None: ...
    @property
    def Length(self) -> float: ...
    @property
    def LengthSquared(self) -> float: ...
    @property
    def X(self) -> float: ...
    @X.setter
    def X(self, value: float) -> float: ...
    @property
    def Y(self) -> float: ...
    @Y.setter
    def Y(self, value: float) -> float: ...
    @staticmethod
    def AngleBetween(vector1: SimVector, vector2: SimVector) -> float: ...
    @staticmethod
    def CrossProduct(vector1: SimVector, vector2: SimVector) -> float: ...
    @staticmethod
    def Determinant(vector1: SimVector, vector2: SimVector) -> float: ...
    @staticmethod
    def Divide(vector: SimVector, scalar: float) -> SimVector: ...
    def GetHashCode(self) -> int: ...
    def Negate(self) -> None: ...
    def Normalize(self) -> None: ...
    @typing.overload
    def __add__(self, vector1: SimVector, vector2: SimVector) -> SimVector: ...
    @typing.overload
    def __add__(self, vector: SimVector, point: SimPoint) -> SimPoint: ...
    def __truediv__(self, vector: SimVector, scalar: float) -> SimVector: ...
    def __eq__(self, vector1: SimVector, vector2: SimVector) -> bool: ...
    # Operator not supported op_Explicit(vector: SimVector)
    # Operator not supported op_Explicit(vector: SimVector)
    def __ne__(self, vector1: SimVector, vector2: SimVector) -> bool: ...
    @typing.overload
    def __mul__(self, scalar: float, vector: SimVector) -> SimVector: ...
    @typing.overload
    def __mul__(self, vector: SimVector, scalar: float) -> SimVector: ...
    @typing.overload
    def __mul__(self, vector1: SimVector, vector2: SimVector) -> float: ...
    @typing.overload
    def __mul__(self, vector: SimVector, matrix: SimMatrix) -> SimVector: ...
    def __sub__(self, vector1: SimVector, vector2: SimVector) -> SimVector: ...
    def __neg__(self, vector: SimVector) -> SimVector: ...
    @staticmethod
    def Parse(source: str) -> SimVector: ...
    @staticmethod
    def Subtract(vector1: SimVector, vector2: SimVector) -> SimVector: ...
    # Skipped Add due to it being static, abstract and generic.

    Add : Add_MethodGroup
    class Add_MethodGroup:
        @typing.overload
        def __call__(self, vector1: SimVector, vector2: SimVector) -> SimVector:...
        @typing.overload
        def __call__(self, vector: SimVector, point: SimPoint) -> SimPoint:...

    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, value: SimVector) -> bool:...
        @typing.overload
        def __call__(self, o: typing.Any) -> bool:...
        @typing.overload
        def __call__(self, vector1: SimVector, vector2: SimVector) -> bool:...

    # Skipped Multiply due to it being static, abstract and generic.

    Multiply : Multiply_MethodGroup
    class Multiply_MethodGroup:
        @typing.overload
        def __call__(self, scalar: float, vector: SimVector) -> SimVector:...
        @typing.overload
        def __call__(self, vector: SimVector, scalar: float) -> SimVector:...
        @typing.overload
        def __call__(self, vector1: SimVector, vector2: SimVector) -> float:...
        @typing.overload
        def __call__(self, vector: SimVector, matrix: SimMatrix) -> SimVector:...

    # Skipped ToString due to it being static, abstract and generic.

    ToString : ToString_MethodGroup
    class ToString_MethodGroup:
        @typing.overload
        def __call__(self) -> str:...
        @typing.overload
        def __call__(self, provider: IFormatProvider) -> str:...



class SimVector3D(IFormattable):
    def __init__(self, x: float, y: float, z: float) -> None: ...
    @property
    def Length(self) -> float: ...
    @property
    def LengthSquared(self) -> float: ...
    @property
    def X(self) -> float: ...
    @X.setter
    def X(self, value: float) -> float: ...
    @property
    def Y(self) -> float: ...
    @Y.setter
    def Y(self, value: float) -> float: ...
    @property
    def Z(self) -> float: ...
    @Z.setter
    def Z(self, value: float) -> float: ...
    @staticmethod
    def AngleBetween(vector1: SimVector3D, vector2: SimVector3D) -> float: ...
    @staticmethod
    def CrossProduct(vector1: SimVector3D, vector2: SimVector3D) -> SimVector3D: ...
    @staticmethod
    def Divide(vector: SimVector3D, scalar: float) -> SimVector3D: ...
    @staticmethod
    def DotProduct(vector1: SimVector3D, vector2: SimVector3D) -> float: ...
    def GetHashCode(self) -> int: ...
    def Negate(self) -> None: ...
    def Normalize(self) -> None: ...
    @typing.overload
    def __add__(self, vector1: SimVector3D, vector2: SimVector3D) -> SimVector3D: ...
    @typing.overload
    def __add__(self, vector: SimVector3D, point: SimPoint3D) -> SimPoint3D: ...
    def __truediv__(self, vector: SimVector3D, scalar: float) -> SimVector3D: ...
    def __eq__(self, vector1: SimVector3D, vector2: SimVector3D) -> bool: ...
    # Operator not supported op_Explicit(vector: SimVector3D)
    # Operator not supported op_Explicit(vector: SimVector3D)
    def __ne__(self, vector1: SimVector3D, vector2: SimVector3D) -> bool: ...
    @typing.overload
    def __mul__(self, scalar: float, vector: SimVector3D) -> SimVector3D: ...
    @typing.overload
    def __mul__(self, vector: SimVector3D, scalar: float) -> SimVector3D: ...
    @typing.overload
    def __mul__(self, vector: SimVector3D, matrix: SimMatrix3D) -> SimVector3D: ...
    @typing.overload
    def __sub__(self, vector1: SimVector3D, vector2: SimVector3D) -> SimVector3D: ...
    @typing.overload
    def __sub__(self, vector: SimVector3D, point: SimPoint3D) -> SimPoint3D: ...
    def __neg__(self, vector: SimVector3D) -> SimVector3D: ...
    @staticmethod
    def Parse(source: str) -> SimVector3D: ...
    # Skipped Add due to it being static, abstract and generic.

    Add : Add_MethodGroup
    class Add_MethodGroup:
        @typing.overload
        def __call__(self, vector1: SimVector3D, vector2: SimVector3D) -> SimVector3D:...
        @typing.overload
        def __call__(self, vector: SimVector3D, point: SimPoint3D) -> SimPoint3D:...

    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, value: SimVector3D) -> bool:...
        @typing.overload
        def __call__(self, o: typing.Any) -> bool:...
        @typing.overload
        def __call__(self, vector1: SimVector3D, vector2: SimVector3D) -> bool:...

    # Skipped Multiply due to it being static, abstract and generic.

    Multiply : Multiply_MethodGroup
    class Multiply_MethodGroup:
        @typing.overload
        def __call__(self, scalar: float, vector: SimVector3D) -> SimVector3D:...
        @typing.overload
        def __call__(self, vector: SimVector3D, scalar: float) -> SimVector3D:...
        @typing.overload
        def __call__(self, vector: SimVector3D, matrix: SimMatrix3D) -> SimVector3D:...

    # Skipped Subtract due to it being static, abstract and generic.

    Subtract : Subtract_MethodGroup
    class Subtract_MethodGroup:
        @typing.overload
        def __call__(self, vector1: SimVector3D, vector2: SimVector3D) -> SimVector3D:...
        @typing.overload
        def __call__(self, vector: SimVector3D, point: SimPoint3D) -> SimPoint3D:...

    # Skipped ToString due to it being static, abstract and generic.

    ToString : ToString_MethodGroup
    class ToString_MethodGroup:
        @typing.overload
        def __call__(self) -> str:...
        @typing.overload
        def __call__(self, provider: IFormatProvider) -> str:...



class SimVector3DConverter(TypeConverter):
    def __init__(self) -> None: ...
    def CanConvertFrom(self, context: ITypeDescriptorContext, sourceType: typing.Type[typing.Any]) -> bool: ...
    def CanConvertTo(self, context: ITypeDescriptorContext, destinationType: typing.Type[typing.Any]) -> bool: ...
    def ConvertFrom(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any) -> typing.Any: ...
    def ConvertTo(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any, destinationType: typing.Type[typing.Any]) -> typing.Any: ...


class SimVectorConverter(TypeConverter):
    def __init__(self) -> None: ...
    def CanConvertFrom(self, context: ITypeDescriptorContext, sourceType: typing.Type[typing.Any]) -> bool: ...
    def CanConvertTo(self, context: ITypeDescriptorContext, destinationType: typing.Type[typing.Any]) -> bool: ...
    def ConvertFrom(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any) -> typing.Any: ...
    def ConvertTo(self, context: ITypeDescriptorContext, culture: CultureInfo, value: typing.Any, destinationType: typing.Type[typing.Any]) -> typing.Any: ...




import typing
from System import EventArgs, Array
from System.Collections import IList, IEnumerable, ICollection, IEnumerator

class INotifyCollectionChanged(typing.Protocol):
    pass


class NotifyCollectionChangedAction(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Add : NotifyCollectionChangedAction # 0
    Remove : NotifyCollectionChangedAction # 1
    Replace : NotifyCollectionChangedAction # 2
    Move : NotifyCollectionChangedAction # 3
    Reset : NotifyCollectionChangedAction # 4


class NotifyCollectionChangedEventArgs(EventArgs):
    @typing.overload
    def __init__(self, action: NotifyCollectionChangedAction) -> None: ...
    @typing.overload
    def __init__(self, action: NotifyCollectionChangedAction, changedItem: typing.Any) -> None: ...
    @typing.overload
    def __init__(self, action: NotifyCollectionChangedAction, changedItem: typing.Any, index: int) -> None: ...
    @typing.overload
    def __init__(self, action: NotifyCollectionChangedAction, changedItem: typing.Any, index: int, oldIndex: int) -> None: ...
    @typing.overload
    def __init__(self, action: NotifyCollectionChangedAction, changedItems: IList) -> None: ...
    @typing.overload
    def __init__(self, action: NotifyCollectionChangedAction, changedItems: IList, index: int, oldIndex: int) -> None: ...
    @typing.overload
    def __init__(self, action: NotifyCollectionChangedAction, changedItems: IList, startingIndex: int) -> None: ...
    @typing.overload
    def __init__(self, action: NotifyCollectionChangedAction, newItem: typing.Any, oldItem: typing.Any) -> None: ...
    @typing.overload
    def __init__(self, action: NotifyCollectionChangedAction, newItem: typing.Any, oldItem: typing.Any, index: int) -> None: ...
    @typing.overload
    def __init__(self, action: NotifyCollectionChangedAction, newItems: IList, oldItems: IList) -> None: ...
    @typing.overload
    def __init__(self, action: NotifyCollectionChangedAction, newItems: IList, oldItems: IList, startingIndex: int) -> None: ...
    @property
    def Action(self) -> NotifyCollectionChangedAction: ...
    @property
    def NewItems(self) -> IList: ...
    @property
    def NewStartingIndex(self) -> int: ...
    @property
    def OldItems(self) -> IList: ...
    @property
    def OldStartingIndex(self) -> int: ...


class StringDictionary(IEnumerable):
    def __init__(self) -> None: ...
    @property
    def Count(self) -> int: ...
    @property
    def IsSynchronized(self) -> bool: ...
    @property
    def Item(self) -> str: ...
    @Item.setter
    def Item(self, value: str) -> str: ...
    @property
    def Keys(self) -> ICollection: ...
    @property
    def SyncRoot(self) -> typing.Any: ...
    @property
    def Values(self) -> ICollection: ...
    def Add(self, key: str, value: str) -> None: ...
    def Clear(self) -> None: ...
    def ContainsKey(self, key: str) -> bool: ...
    def ContainsValue(self, value: str) -> bool: ...
    def CopyTo(self, array: Array, index: int) -> None: ...
    def GetEnumerator(self) -> IEnumerator: ...
    def Remove(self, key: str) -> None: ...




import os
import sys
import colorlog
from ruamel.yaml import YAML, add_representer

from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from .data_model import DataModel
    from .object_mapper import PythonMapper

def setup_logging():
    handler = colorlog.StreamHandler()
    formatter = colorlog.ColoredFormatter(
        "%(log_color)s%(levelname)-8s%(reset)s %(blue)s%(message)s",
        datefmt=None,
        reset=True,
        log_colors={
            'DEBUG':    'cyan',
            'INFO':     'green',
            'WARNING':  'yellow',
            'ERROR':    'red',
            'CRITICAL': 'red,bg_white',
        },
        secondary_log_colors={},
        style='%'
    )

    handler.setFormatter(formatter)

    logger = colorlog.getLogger('PySimultan')
    logger.addHandler(handler)

    return logger


def setup():
    import colorlog
    try:
        import importlib.resources as pkg_resources
    except ImportError:
        # Try backported to PY<37 importlib_resources.
        import importlib_resources as pkg_resources

    from . import resources
    sys.path.append(str(pkg_resources.files(resources)))

    logger = colorlog.getLogger('PySimultan')
    logger.setLevel('DEBUG')

    dll_path = os.environ.get('SIMULTAN_SDK_DIR', None)
    dll_path = None
    if dll_path is None:
        with pkg_resources.path(resources, 'SIMULTAN.dll') as r_path:
            dll_path = str(r_path)
    sys.path.append(dll_path)

    from pythonnet import load
    from pythonnet import clr_loader, set_runtime
    list(clr_loader.find_runtimes())
    load('coreclr')
    import clr
    test = clr.AddReference(
        os.path.join(dll_path, 'SIMULTAN.dll') if not dll_path.endswith('SIMULTAN.dll') else dll_path)
    clr.AddReference("System.Security.Cryptography")
    # clr.AddReference(os.path.join(dll_path, 'SIMULTAN'))

    from SIMULTAN.Data.Components import SimComponent

    continue_on_error = True

    def represent_none(self, _):
        return self.represent_scalar('tag:yaml.org,2002:null', '')

    add_representer(type(None), represent_none)
    yaml = YAML()
    yaml.default_flow_style = None
    yaml.preserve_quotes = True
    yaml.allow_unicode = True

    return yaml


logger = setup_logging()
yaml = setup()


class Config:
    def __init__(self):
        self._default_data_model = None
        self._default_mapper = None
        self.logger = logger

    def get_default_data_model(self, *args, **kwargs) -> 'DataModel':
        return self._default_data_model

    def get_default_mapper(self, *args, **kwargs) -> 'PythonMapper':
        return self._default_mapper

    def set_default_data_model(self, data_model: 'DataModel'):
        self.logger.debug(f'set_default_data_model: {id(data_model)}')
        self._default_data_model = data_model

    def set_default_mapper(self, mapper: 'PythonMapper'):
        self.logger.debug(f'set_default_mapper: {id(mapper)}')
        self._default_mapper = mapper


config = Config()

from .data_model import DataModel
from .files import FileInfo
from .object_mapper import PythonMapper
from .taxonomy_maps import Content, TaxonomyMap



from src.PySimultan2.data_model import DataModel
from src.PySimultan2.utils import create_taxonomy, get_ot_create_taxonomy_entry

try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 importlib_resources.
    import importlib_resources as pkg_resources


import PySimultan2.tests.readme_examples.resources as readme_examples

with pkg_resources.path(readme_examples, 'empty_project.simultan') as r_path:
    project_file = str(r_path)

data_model = DataModel(project_path=project_file)

taxonomy = create_taxonomy('test_name',
                           'test_key',
                           'test_description',
                           data_model=data_model)

taxonomy_entry = get_ot_create_taxonomy_entry('test_taxonomy_entry_name',
                                       'test_taxonomy_entry_key',
                                       'test_taxonomy_entry_description',
                                              sim_taxonomy=taxonomy)

data_model.save()
data_model.cleanup()



import typing, abc
from SIMULTAN.Data.SimMath import SimColor
from System.Collections.Generic import IEnumerable_1
from System.ComponentModel import INotifyPropertyChanged
from System.Collections.ObjectModel import ObservableCollection_1
from SIMULTAN.Data import SimNamedObject_1, SimId, SimManagedCollection_1, IReferenceLocation
from SIMULTAN.Data.MultiValues import SimMultiValueBigTable
from System import Guid, MulticastDelegate, IAsyncResult, AsyncCallback, DateTime
from System.Reflection import MethodInfo
from SIMULTAN.Projects import ProjectData

class DefaultColorMappingColors:
    def __init__(self) -> None: ...
    BaseLightingFactor : float
    NotParameterizedColor : SimColor
    OutOfRangeColor : SimColor


class SimAveragePrefilter(SimPrefilter):
    def __init__(self) -> None: ...
    @property
    def Owner(self) -> SimValueMapping: ...
    @Owner.setter
    def Owner(self, value: SimValueMapping) -> SimValueMapping: ...
    def Filter(self, values: IEnumerable_1[float], timelineCurrentIndex: int) -> IEnumerable_1[float]: ...


class SimColorMap(abc.ABC):
    @property
    def Owner(self) -> SimValueMapping: ...
    @Owner.setter
    def Owner(self, value: SimValueMapping) -> SimValueMapping: ...
    @abc.abstractmethod
    def Map(self, value: float) -> SimColor: ...


class SimColorMarker(INotifyPropertyChanged):
    def __init__(self, value: float, color: SimColor) -> None: ...
    @property
    def Color(self) -> SimColor: ...
    @Color.setter
    def Color(self, value: SimColor) -> SimColor: ...
    @property
    def Value(self) -> float: ...
    @Value.setter
    def Value(self, value: float) -> float: ...


class SimColorMarkerCollection(ObservableCollection_1[SimColorMarker]):
    @typing.overload
    def __init__(self, owner: SimColorMap) -> None: ...
    @typing.overload
    def __init__(self, owner: SimColorMap, marker: IEnumerable_1[SimColorMarker]) -> None: ...
    @property
    def Count(self) -> int: ...
    @property
    def Item(self) -> SimColorMarker: ...
    @Item.setter
    def Item(self, value: SimColorMarker) -> SimColorMarker: ...
    @property
    def Owner(self) -> SimColorMap: ...


class SimComponentIndexUsage(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Row : SimComponentIndexUsage # 0
    Column : SimComponentIndexUsage # 1


class SimDefaultPrefilter(SimPrefilter):
    def __init__(self) -> None: ...
    @property
    def Owner(self) -> SimValueMapping: ...
    @Owner.setter
    def Owner(self, value: SimValueMapping) -> SimValueMapping: ...
    def Filter(self, values: IEnumerable_1[float], timelineCurrentIndex: int) -> IEnumerable_1[float]: ...


class SimLinearGradientColorMap(SimColorMap):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, marker: IEnumerable_1[SimColorMarker]) -> None: ...
    @property
    def ColorMarkers(self) -> SimColorMarkerCollection: ...
    @property
    def Owner(self) -> SimValueMapping: ...
    @Owner.setter
    def Owner(self, value: SimValueMapping) -> SimValueMapping: ...
    @staticmethod
    def Lerp(a: SimColorMarker, b: SimColorMarker, val: float) -> SimColor: ...
    def Map(self, value: float) -> SimColor: ...


class SimMaximumPrefilter(SimPrefilter):
    def __init__(self) -> None: ...
    @property
    def Owner(self) -> SimValueMapping: ...
    @Owner.setter
    def Owner(self, value: SimValueMapping) -> SimValueMapping: ...
    def Filter(self, values: IEnumerable_1[float], timelineCurrentIndex: int) -> IEnumerable_1[float]: ...


class SimMinimumPrefilter(SimPrefilter):
    def __init__(self) -> None: ...
    @property
    def Owner(self) -> SimValueMapping: ...
    @Owner.setter
    def Owner(self, value: SimValueMapping) -> SimValueMapping: ...
    def Filter(self, values: IEnumerable_1[float], timelineCurrentIndex: int) -> IEnumerable_1[float]: ...


class SimPrefilter(abc.ABC):
    @property
    def Owner(self) -> SimValueMapping: ...
    @Owner.setter
    def Owner(self, value: SimValueMapping) -> SimValueMapping: ...
    @abc.abstractmethod
    def Filter(self, values: IEnumerable_1[float], timelineCurrentIndex: int) -> IEnumerable_1[float]: ...


class SimThresholdColorMap(SimColorMap):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, marker: IEnumerable_1[SimColorMarker]) -> None: ...
    @property
    def ColorMarkers(self) -> SimColorMarkerCollection: ...
    @property
    def Owner(self) -> SimValueMapping: ...
    @Owner.setter
    def Owner(self, value: SimValueMapping) -> SimValueMapping: ...
    def Map(self, value: float) -> SimColor: ...


class SimValueMapping(SimNamedObject_1[SimValueMappingCollection]):
    def __init__(self, name: str, table: SimMultiValueBigTable, prefilter: SimPrefilter, colorMap: SimColorMap) -> None: ...
    @property
    def ColorMap(self) -> SimColorMap: ...
    @ColorMap.setter
    def ColorMap(self, value: SimColorMap) -> SimColorMap: ...
    @property
    def ComponentIndexUsage(self) -> SimComponentIndexUsage: ...
    @ComponentIndexUsage.setter
    def ComponentIndexUsage(self, value: SimComponentIndexUsage) -> SimComponentIndexUsage: ...
    @property
    def Description(self) -> str: ...
    @Description.setter
    def Description(self, value: str) -> str: ...
    @property
    def Factory(self) -> SimValueMappingCollection: ...
    @Factory.setter
    def Factory(self, value: SimValueMappingCollection) -> SimValueMappingCollection: ...
    @property
    def GlobalID(self) -> Guid: ...
    @property
    def Id(self) -> SimId: ...
    @Id.setter
    def Id(self, value: SimId) -> SimId: ...
    @property
    def LocalID(self) -> int: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...
    @property
    def Prefilter(self) -> SimPrefilter: ...
    @Prefilter.setter
    def Prefilter(self, value: SimPrefilter) -> SimPrefilter: ...
    @property
    def Table(self) -> SimMultiValueBigTable: ...
    @Table.setter
    def Table(self, value: SimMultiValueBigTable) -> SimMultiValueBigTable: ...
    def ApplyMapping(self, objectIndex: int, timelineIndex: int) -> SimColor: ...

    class ValueMappingChangedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any) -> None: ...



class SimValueMappingCollection(SimManagedCollection_1[SimValueMapping]):
    def __init__(self, projectData: ProjectData) -> None: ...
    @property
    def CalledFromLocation(self) -> IReferenceLocation: ...
    @property
    def Count(self) -> int: ...
    @property
    def HasChanges(self) -> bool: ...
    @property
    def Item(self) -> SimValueMapping: ...
    @Item.setter
    def Item(self, value: SimValueMapping) -> SimValueMapping: ...
    @property
    def LastChange(self) -> DateTime: ...
    @property
    def ProjectData(self) -> ProjectData: ...
    def EndLoading(self) -> None: ...
    def StartLoading(self) -> None: ...




import typing, abc
from SIMULTAN.Data.MultiValues import SimMultiValueBigTable, SimMultiValueCollection, SimMultiValueBigTableHeader
from System.Collections.Generic import List_1
from System import Exception, Array_1
from System.Collections import IDictionary
from System.Reflection import MethodBase
from System.IO import FileInfo
from System.ComponentModel import ISynchronizeInvoke
from System.Text import Encoding

class BigTableToCSVExporter(abc.ABC):
    @staticmethod
    def ExportToCSV(table: SimMultiValueBigTable, filePathAndName: str, delimiter: str, decimalSeparator: str, rowHeaderHeader: str, columnHeaders: bool, rowHeaders: bool, columnUnits: bool, rowUnits: bool) -> None: ...


class CSVExporter:
    def __init__(self, nrColumns: int, csvDelimiter: str = ...) -> None: ...
    CSV_LINEDELIMITER : str
    @property
    def CSV_DELIMITER(self) -> str: ...
    @CSV_DELIMITER.setter
    def CSV_DELIMITER(self, value: str) -> str: ...
    def AddMultipleRecords(self, records: List_1[List_1[str]]) -> None: ...
    def AddRecord(self, record: List_1[str]) -> None: ...
    def WriteFile(self, filePath: str) -> None: ...


class CSVImportException(Exception):
    def __init__(self, reason: CSVImportExceptionReason, data: Array_1[typing.Any]) -> None: ...
    @property
    def Data(self) -> IDictionary: ...
    @property
    def HelpLink(self) -> str: ...
    @HelpLink.setter
    def HelpLink(self, value: str) -> str: ...
    @property
    def HResult(self) -> int: ...
    @HResult.setter
    def HResult(self, value: int) -> int: ...
    @property
    def InnerException(self) -> Exception: ...
    @property
    def Message(self) -> str: ...
    @property
    def Reason(self) -> CSVImportExceptionReason: ...
    @property
    def ReasonData(self) -> Array_1[typing.Any]: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> str: ...
    @property
    def StackTrace(self) -> str: ...
    @property
    def TargetSite(self) -> MethodBase: ...


class CSVImportExceptionReason(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    EmptyFile : CSVImportExceptionReason # 0
    NotEnoughRows : CSVImportExceptionReason # 1
    NotEnoughColumns : CSVImportExceptionReason # 2
    ColumnCountMismatch : CSVImportExceptionReason # 3
    ParseError : CSVImportExceptionReason # 4


class CSVToValueFieldImporter(abc.ABC):
    @staticmethod
    def ConvertCsvToMultiValueBigTableComponents(file: FileInfo, settings: CSVToValueFieldImporterSettings, rowHeaderFormat: str, rowHeaderUnit: str, columnHeaderFormat: str, columnHeaderUnit: str) -> MultiValueBigTableComponents: ...
    @staticmethod
    def Import(collection: SimMultiValueCollection, file: FileInfo, settings: CSVToValueFieldImporterSettings, unitRows: str, unitColumns: str, rowHeaderFormat: str, rowHeaderUnit: str, columnHeaderFormat: str, columnHeaderUnit: str, dispatcher: ISynchronizeInvoke = ...) -> SimMultiValueBigTable: ...


class CSVToValueFieldImporterSettings:
    def __init__(self) -> None: ...
    @property
    def DecimalSeparator(self) -> str: ...
    @DecimalSeparator.setter
    def DecimalSeparator(self, value: str) -> str: ...
    @property
    def Delimiter(self) -> str: ...
    @Delimiter.setter
    def Delimiter(self, value: str) -> str: ...
    @property
    def HasColumnHeaders(self) -> bool: ...
    @HasColumnHeaders.setter
    def HasColumnHeaders(self, value: bool) -> bool: ...
    @property
    def HasColumnUnits(self) -> bool: ...
    @HasColumnUnits.setter
    def HasColumnUnits(self, value: bool) -> bool: ...
    @property
    def HasRowHeaders(self) -> bool: ...
    @HasRowHeaders.setter
    def HasRowHeaders(self, value: bool) -> bool: ...
    @property
    def HasRowUnits(self) -> bool: ...
    @HasRowUnits.setter
    def HasRowUnits(self, value: bool) -> bool: ...
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> str: ...


class FileHelper(abc.ABC):
    @staticmethod
    def GetEncoding(filename: str) -> Encoding: ...


class MultiValueBigTableComponents:
    def __init__(self, columnHeaders: List_1[SimMultiValueBigTableHeader], rowHeaders: List_1[SimMultiValueBigTableHeader], values: List_1[List_1[typing.Any]]) -> None: ...
    ColumnHeaders : List_1[SimMultiValueBigTableHeader]
    RowHeaders : List_1[SimMultiValueBigTableHeader]
    Values : List_1[List_1[typing.Any]]




from PySimultan2.src.PySimultan2.data_model import DataModel
from PySimultan2.src.PySimultan2.object_mapper import PythonMapper
from PySimultan2.src.PySimultan2.taxonomy_maps import TaxonomyMap, Content

import resources

try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 importlib_resources.
    import importlib_resources as pkg_resources


with pkg_resources.path(resources, 'test_set_dictionary.simultan') as r_path:
    project_path = str(r_path)


data_model = DataModel.create_new_project(project_path=project_path,
                                          user_name='admin',
                                          password='admin')

mapper = PythonMapper()


def map_classes():
    class TestComponent(object):
        def __init__(self, *args, **kwargs):
            self.mapped_property = kwargs.get('mapped_property')
            self.value2 = kwargs.get('value2')

    content0 = Content(text_or_key='value',  # text or key of the content/parameter/property
                       property_name='mapped_property',  # name of the generated property
                       type=None,  # type of the content/parameter/property
                       unit=None,  # unit of the content/parameter/property
                       documentation='mapped_property to test',
                       component_policy='subcomponent')

    content1 = Content(text_or_key='value2',  # text or key of the content/parameter/property
                       property_name='value2',  # name of the generated property
                       type=None,  # type of the content/parameter/property
                       unit=None,  # unit of the content/parameter/property
                       documentation='value2',
                       component_policy='reference')

    test_component_map = TaxonomyMap(taxonomy_name='PySimultan',
                                     taxonomy_key='PySimultan',
                                     taxonomy_entry_name='TestComponent',
                                     taxonomy_entry_key='TestComponent',
                                     content=[content0, content1],
                                     )

    mapper.register(test_component_map.taxonomy_entry_key, TestComponent, taxonomy_map=test_component_map)
    cls1 = mapper.get_mapped_class(test_component_map.taxonomy_entry_key)

    test_component_map_2 = TaxonomyMap(taxonomy_name='PySimultan',
                                       taxonomy_key='PySimultan',
                                       taxonomy_entry_name='TestComponent2',
                                       taxonomy_entry_key='TestComponent2',
                                       content=[content0],
                                       )

    mapper.register(test_component_map_2.taxonomy_entry_key, TestComponent, taxonomy_map=test_component_map_2)
    cls2 = mapper.get_mapped_class(test_component_map_2.taxonomy_entry_key)

    return cls1, cls2


mapped_cls_1, mapped_cls_2 = map_classes()
component = mapped_cls_1(name='test_component',
                         mapped_property=None)


def test_set_dictionary_param(component):

    other_component = mapped_cls_2(name='other_component',
                                   mapped_property=None)
    other_component_2 = mapped_cls_2(name='other_component_2',
                                     mapped_property=None)
    other_component_3 = mapped_cls_2(name='other_component_3',
                                     mapped_property=None)
    other_component_4 = mapped_cls_2(name='other_component_4',
                                     mapped_property=None)

    new_test = mapped_cls_1(name='test_component_2',
                            mapped_property={'val_a': 1,
                                             'val_b': 2.0,
                                             'val_c': 'test',
                                             'val_d': True,
                                             'val_e': [other_component, other_component_2, other_component_3],
                                             'val_f': {'a': other_component_2, 'b': other_component_3},
                                             'val_g': None,
                                             'val_h': other_component,
                                             'val_i': other_component_4,
                                             })

    test_dict = {'val_a': 1,
                 'val_b': 2.0,
                 'val_c': 'test',
                 'val_d': True,
                 'val_e': [other_component, other_component_2, other_component_3],
                 'val_f': {'a': other_component_2, 'b': other_component_3},
                 'val_g': None,
                 'val_h': other_component,
                 'val_i': 5,
                 'val_j': new_test
                 }

    component.mapped_property = test_dict
    component.mapped_property['val_a'] = 222

    assert component.mapped_property['val_j'].mapped_property['val_a'] == 1
    component.mapped_property['val_j'].mapped_property._generate_internal_dict()

    component.mapped_property['val_i'] = other_component_4

    print(component.mapped_property)


def load_data_model(data_model: DataModel):
    data_model = DataModel(project_path=project_path,
                           user_name='admin',
                           password='admin')
    typed_data = data_model.get_typed_data(mapper=mapper,
                                           create_all=True)

    TestComponent = mapper.get_mapped_class('TestComponent')
    test_component = TestComponent.cls_instances[0]

    TestComponent2 = mapper.get_mapped_class('TestComponent2')
    other_component = next(x for x in TestComponent2.cls_instances if x.name == 'other_component')
    other_component_2 = next(x for x in TestComponent2.cls_instances if x.name == 'other_component_2')
    other_component_3 = next(x for x in TestComponent2.cls_instances if x.name == 'other_component_3')
    other_component_4 = next(x for x in TestComponent2.cls_instances if x.name == 'other_component_4')

    assert test_component.mapped_property['val_a'] == 222
    assert test_component.mapped_property['val_b'] == 2.0
    assert test_component.mapped_property['val_c'] == 'test'
    assert test_component.mapped_property['val_d'] is True
    assert test_component.mapped_property['val_g'] is None
    assert test_component.mapped_property['val_h'] == other_component
    assert len(test_component.mapped_property['val_e']) == 3
    assert test_component.mapped_property['val_e'][0] == other_component
    assert test_component.mapped_property['val_e'][1] == other_component_2
    assert test_component.mapped_property['val_e'][2] == other_component_3
    assert test_component.mapped_property['val_f']['a'] == other_component_2
    assert test_component.mapped_property['val_f']['b'] == other_component_3
    assert test_component.mapped_property['val_i'] == other_component_4

    return data_model


if __name__ == '__main__':
    test_set_dictionary_param(component)
    data_model.save()
    mapper.clear()
    data_model.cleanup()
    data_model = DataModel(project_path=project_path,
                           user_name='admin',
                           password='admin')
    load_data_model(data_model)


data_model.cleanup()
print('Test passed')
